import random
from pathlib import Path
from collections.abc import Callable
from .horse import Horse, Event, Event_list
from .start import load_dlcs
from clovers.core.config import config as clovers_config
from .config import Config

config_key = __package__
config_data = Config.model_validate(clovers_config.get(config_key, {}))
clovers_config[config_key] = config_data.model_dump()

track_length = config_data.setting_track_length
base_move_range = config_data.base_move_range
random_move_min, random_move_max = config_data.random_move_range
random_move_range = int(random_move_min * track_length), int(random_move_max * track_length)
range_of_player_numbers = config_data.range_of_player_numbers


class RaceWorld:
    event_list: list[Event] = load_dlcs()

    @classmethod
    def update_event_list(cls, resource_path: Path):
        cls.event_list = load_dlcs(resource_path)

    def __init__(self):
        self.racetrack: list[Horse] = []
        """赛马场跑道"""
        self.track_length = track_length
        """跑道长度"""
        self.base_move_range = base_move_range
        """随机事件跳转范围"""
        self.random_move_range = random_move_range
        """随机事件跳转范围"""
        self.min_player_numbers, self.max_player_numbers = range_of_player_numbers
        """赛马场容量"""
        self.status: int = 0
        """
        状态指示器
            0:马儿进场未开始
            1:开始
            2:暂停
        """
        self.round: int = 0
        """当前回合数"""
        self.only_keys = set()
        """唯一事件记录"""
        self.event_randvalue = config_data.event_randvalue
        """事件触发随机值（千分数，比如 450 即触发概率 450‰）"""

    def join_horse(self, horse_name: str, user_id: str, user_name: str, location=0, round=0):
        """
        增加赛马位
        """
        if self.status != 0:
            return
        l = len(self.racetrack)
        if l >= self.max_player_numbers:
            return "> 加入失败！\n> 赛马场就那么大，满了满了！"
        if any(horse.playeruid == user_id for horse in self.racetrack):
            return "> 加入失败！\n> 您已经加入了赛马场!"
        self.racetrack.append(Horse(horse_name, user_id, user_name, location, round))
        horse_name = horse_name[:1] + "酱" if len(horse_name) >= 7 else horse_name
        return f"> {user_name} 加入赛马成功\n> 赌上马儿性命的一战即将开始!\n> 赛马场位置:{l + 1}/{self.max_player_numbers}"

    def event_main(self, horse: Horse, event: Event, event_delay_key=0):
        # 该马儿是否死亡/离开/眩晕，死亡则结束事件链
        if event_delay_key == 0 and (horse.is_die or horse.is_away or horse.find_buff("vertigo")):
            return
        # 读取事件限定值
        if only_key := event.only_key:
            if only_key in self.only_keys:
                return
            else:
                self.only_keys.add(only_key)
        # 读取 target 目标，计算<0><1>， target_name_0 ， target_name_1
        target_name_0 = horse.horse
        match event.target:
            case 0:
                targets = [horse]
                target_name_1 = target_name_0
            case 1:
                target = random.choice([x for x in self.racetrack if not x is horse])
                targets = [target]
                target_name_1 = target.horse
            case 2:
                targets = self.racetrack
                target_name_1 = "所有马儿"
            case 3:
                targets = [x for x in self.racetrack if not x is horse]
                target_name_1 = "其他所有马儿"
            case 4:
                target = random.choice(self.racetrack)
                targets = [target]
                target_name_1 = target.horse
            case 5:
                target = random.choice([x for x in self.racetrack if not x is horse])
                targets = [target, horse]
                target_name_1 = target.horse
            case 6:
                index = self.racetrack.index(horse)
                side = [x for x in [index + 1, index - 1] if 0 <= x < len(self.racetrack)]
                target = self.racetrack[random.choice(side)]
                targets = [target]
                target_name_1 = target.horse
            case 7:
                index = self.racetrack.index(horse)
                side = [x for x in [index + 1, index - 1] if 0 <= x < len(self.racetrack)]
                targets = [self.racetrack[i] for i in side]
                target_name_1 = f"在{horse.horse}两侧的马儿"
            case _:
                return
        # 判定 target_is_buff
        if event.target_is_buff:
            targets = [x for x in targets if x.find_buff(event.target_is_buff)]
        # 判定 target_no_buff
        if event.target_is_buff:
            targets = [x for x in targets if not x.find_buff(event.target_is_buff)]
        # 无目标则结束事件
        if not targets:
            return
        # 读取 event_name
        event_name = event.event_name
        # 读取 describe 事件描述
        print(f"执行事件: {event_name}")
        print(f"<0>为：{target_name_0}，<1>为：{target_name_1}")
        # 读取 describe 事件描述
        describe: list[str | None] = [event.describe.replace("<0>", target_name_0).replace("<1>", target_name_1)]

        def action(targets: list[Horse], callback: Callable[..., None], *args):
            for horse in targets:
                callback(horse, *args)

        """===============以下为一次性事件==============="""
        if event.live == 1:
            action(targets, lambda horse: horse.del_buff("die"))
        if event.move:
            action(targets, lambda horse, move: horse.location_move(move), event.move)
        if not event.track_to_location is None:
            action(targets, lambda horse, move_to: horse.location_to(move_to), event.track_to_location)
        if event.track_random_location == 1:
            action(
                targets,
                lambda horse, random_move_range: horse.location_to(random.randint(*random_move_range)),
                self.random_move_range,
            )
        if event.buff_time_add:
            action(targets, lambda horse, time_add: horse.buff_addtime(time_add), event.buff_time_add)
        if event.del_buff:
            action(targets, lambda horse, del_buff: horse.del_buff(del_buff), event.del_buff)
        if event.track_exchange_location == 1 and event.target in {1, 6}:
            # 马儿互换位置
            target = targets[0]
            location = target.location, horse.location
            horse.location_to(location[0])
            target.location_to(location[1])
        if random_event_once := event.random_event_once:
            action(
                targets,
                lambda horse, describe, event_list: describe.append(self.event_main(horse, self.roll_event(event_list), 1)),
                describe,
                random_event_once,
            )
        """===============以下为永久事件==============="""
        if event.die == 1:
            action(targets, lambda horse, die_name: horse.add_buff(die_name, {"die"}, 1, 9999), event.die_name)
        if event.away == 1:
            action(targets, lambda horse, away_name: horse.add_buff(away_name, {"away"}, 1, 9999), event.away_name)
        # ==============================连锁事件预留位置，暂时没做
        # 连锁事件，以后大概也没了 ——karis

        """===============以下为buff事件==============="""
        if event.rounds:

            def add_buff(
                horse: Horse,
                buff_name: str,
                buffs: set[str],
                round_start: int,
                round_end: int,
                move_min: int = 0,
                move_max: int = 0,
                event_in_buff: Event_list = [],
            ):
                horse.add_buff(buff_name, buffs, round_start, round_end, move_min, move_max, event_in_buff)

            action(
                targets,
                lambda horse, *args: add_buff(horse, *args),
                event.name,
                event.buffs,
                self.round + 1,
                self.round + event.rounds,
                event.move_min,
                event.move_max,
                event.random_event,
            )
        """===============以下为延迟事件==============="""
        if delay_event := event.delay_event:
            delay_rounds, delay_event = delay_event
            if delay_rounds > 1:
                action(
                    targets,
                    lambda horse, delay_event: horse.delay_events.append((self.round + delay_rounds, delay_event)),
                    delay_event,
                )
        if delay_event_self := event.delay_event_self:
            delay_rounds, delay_event = delay_event_self
            if delay_rounds > 1:
                horse.delay_events.append((self.round + delay_rounds, delay_event))
        """===============以下同步事件==============="""
        if another_event := event.another_event:
            action(
                targets,
                lambda horse, describe, event: describe.append(self.event_main(horse, event, 1)),
                describe,
                another_event,
            )
        if another_event_self := event.another_event_self:
            describe.append(self.event_main(horse, another_event_self, 1))

        """==========永久事件2，换赛道/加马=========="""
        if add_horse := event.add_horse:
            horse_name = add_horse.get("horsename", "一只马")
            horse_uid = add_horse.get("uid", "None")
            horse_owner = add_horse.get("owner", "神秘生物")
            horse_location = add_horse.get("location", 0)
            self.join_horse(horse_name, horse_uid, horse_owner, horse_location, self.round)
        if (replace_horse := event.replace_horse) and len(targets) == 1:
            horse_name = replace_horse.get("horsename", "一只马")
            horse_uid = replace_horse.get("uid", "None")
            horse_owner = replace_horse.get("owner", "神秘生物")
            targets[0].__init__(horse_name, horse_uid, horse_owner, round=self.round)
        return "\n".join(x for x in describe if x)

    @staticmethod
    def roll_event(event_list: Event_list) -> Event:
        weights = []
        events = []
        before_randvalue = 0
        for randvalue, event in event_list:
            weights.append(randvalue - before_randvalue)
            before_randvalue = randvalue
            events.append(event)
        return random.choices(events, weights, k=1)[0]

    def nextround(self):
        """
        回合开始，回合数+1
        """
        self.round += 1
        event_log: list[str | None] = []
        for horse in self.racetrack:
            horse.round = self.round
            horse.location_add_move = 0
            # 移除超时buff
            horse.buff = [buff for buff in horse.buff if buff.round_end >= self.round]
            # 延时事件触发
            for delay_round, delay_event in horse.delay_events:
                if self.round == delay_round:
                    event_log.append(self.event_main(horse, delay_event, 1))
            horse.delay_events = [(delay_round, delay_event) for delay_round, delay_event in horse.delay_events if delay_round > self.round]
            # buff随机事件触发
            for buff in horse.buff:
                if buff.event_in_buff:
                    buff_event = self.roll_event(buff.event_in_buff)
                    event_log.append(self.event_main(horse, buff_event, 1))
            # 马儿移动,包含死亡/离开/止步判定
            if horse.is_die or horse.is_away:
                base_move = 0
            else:
                if horse.is_stop:  # 止步不影响主动事件
                    base_move = 0
                else:
                    base_move = horse.base_move(*self.base_move_range)
                # 随机事件判定
                if random.randint(1, 1000) <= self.event_randvalue:
                    event = random.choice(self.event_list)
                    event_log.append(self.event_main(horse, event))
            horse.move(base_move, self.track_length)
        return "\n".join(x for x in event_log if x)

    def is_die_all(self) -> bool:
        """
        所有马儿是否死亡/离开
        """
        return all(horse.is_die or horse.is_away for horse in self.racetrack)



import time
import heapq
import math
import random
from io import BytesIO
from collections import Counter
from clovers_apscheduler import scheduler
from clovers.utils.tools import gini_coef, format_number
from clovers_leafgame.core.clovers import Event, Rule
from clovers_leafgame.core.data import Account, Group, Stock
from clovers_leafgame.main import plugin, manager
from clovers_leafgame.item import GOLD, LICENSE, STD_GOLD, REVOLUTION_MARKING, item_name_rule
from clovers_leafgame.output import text_to_image, endline, invest_card, prop_card
from clovers.core.config import config as clovers_config
from .config import Config

config_key = __package__
config_data = Config.model_validate(clovers_config.get(config_key, {}))
clovers_config[config_key] = config_data.model_dump()


revolt_gold = config_data.revolt_gold
revolt_gini = config_data.revolt_gini
gini_filter_gold = config_data.gini_filter_gold
revolt_cd = config_data.revolt_cd
company_public_gold = config_data.company_public_gold


@plugin.handle(["发起重置"], ["group_id"])
async def _(event: Event):
    group_id = event.group_id
    group = manager.data.group(group_id)
    revolution_time = group.extra.get("revolution_time", 0)
    if time.time() - revolution_time < revolt_cd:
        return f"重置正在冷却中，结束时间：{time.strftime('%H:%M:%S', time.localtime(revolution_time + revolt_cd))}"
    ranklist: list[tuple[Account, int]] = []
    sum_wealths = 0
    for account_id in group.accounts_map.values():
        account = manager.data.account_dict[account_id]
        n = account.bank[GOLD.id]
        if account.bank[GOLD.id] >= gini_filter_gold:
            ranklist.append((account, n))
        sum_wealths += n
    if sum_wealths < company_public_gold:
        return f"本群金币（{sum_wealths}）小于{company_public_gold}，未满足重置条件。"
    gini = gini_coef([x[1] for x in ranklist])
    if gini < revolt_gini:
        return f"当前基尼系数为{round(gini,3)}，未满足重置条件。"
    ranklist = heapq.nlargest(10, ranklist, key=lambda x: x[1])
    top = ranklist[0][0]
    REVOLUTION_MARKING.locate_bank(*manager.locate_account(top.user_id, group_id))[REVOLUTION_MARKING.id] += 1
    group.extra["revolution_time"] = time.time()
    revolution_achieve: dict = group.extra.setdefault("revolution_achieve", {})
    revolution_achieve[top.user_id] = revolution_achieve.get(top.user_id, 0) + 1
    for i, (account, n) in enumerate(ranklist):
        account.bank[GOLD.id] = int(n * i / 10)
    for account_id in group.accounts_map.values():
        manager.data.account_dict[account_id].extra["revolution"] = False
    rate = group.level / group.level + 1
    for prop_id, n in group.bank.items():
        prop = manager.props_library[prop_id]
        if prop.domain == 1:
            group.bank[prop_id] = int(n * rate)
    group.level += 1
    return f"当前系数为：{round(gini,3)}，重置成功！恭喜{top.name}进入挂件榜☆！重置签到已刷新。"


@plugin.handle(["重置签到", "领取金币"], ["user_id", "group_id", "nickname", "avatar"])
async def _(event: Event):
    user, account = manager.account(event)
    if avatar := event.avatar:
        user.avatar_url = avatar
    extra = account.extra
    if not extra.setdefault("revolution", True):
        return "你没有待领取的金币"
    n = random.randint(*revolt_gold)
    GOLD.deal(account.bank, n)
    extra["revolution"] = False
    return f"这是你重置后获得的金币！你获得了 {n} 金币"


@plugin.handle(["金币转入"], ["user_id", "group_id", "nickname"])
async def _(event: Event):
    n = event.args_to_int()
    if n == 0:
        return
    user, account = manager.account(event)
    level = manager.data.group(event.group_id).level
    if n > 0:
        n_out = n * level
        if n_out > user.bank[STD_GOLD.id]:
            n_out = user.bank[STD_GOLD.id]
            n_in = int(user.bank[STD_GOLD.id] / level)
        else:
            n_in = n
        user.bank[STD_GOLD.id] -= n_out
        account.bank[GOLD.id] += n_in
        return f"你成功将{n_out}枚标准金币兑换为{n_in}枚金币"
    else:
        n_out = -n
        if n_out > account.bank[GOLD.id]:
            n_out = account.bank[GOLD.id]
            n_in = account.bank[GOLD.id] * level
        else:
            n_in = n_out * level
        user.bank[STD_GOLD.id] += n_in
        account.bank[GOLD.id] -= n_out
        return f"你成功将{n_out}枚金币兑换为{n_in}枚标准金币"


@plugin.handle(["金币转移"], ["user_id", "group_id"])
async def _(event: Event):
    if not (args := event.args_parse()):
        return
    group_name, xfer = args[:2]
    group_in = manager.group_library.get(group_name)
    if not group_in:
        return f"没有 {group_name} 的注册信息"
    user = manager.data.user(event.user_id)
    if not (receiver_account_id := user.accounts_map.get(group_in.id)):
        return f"你在{group_in.nickname}没有帐户"
    group_out = manager.data.group(event.group_id or user.connect)
    if not (sender_account_id := user.accounts_map.get(group_out.id)):
        return "你在本群没有帐户"
    if (n := group_out.bank[STD_GOLD.id]) < company_public_gold:
        return f"本群金币过少（{n}<{company_public_gold}），无法完成结算"
    if (n := group_in.bank[STD_GOLD.id]) < company_public_gold:
        return f"【{group_in.nickname}】金币过少（{n}<{company_public_gold}），无法完成结算"
    bank_in = manager.data.account_dict[receiver_account_id].bank
    bank_out = manager.data.account_dict[sender_account_id].bank
    if xfer < 0:
        xfer = -xfer
        bank_out, bank_in = bank_in, bank_out
        group_out, group_in = group_in, group_out
    ExRate = group_out.level / group_in.level
    receipt = xfer * ExRate
    if receipt < 1:
        return f"转入金币{round(receipt,2)}不可小于1枚（汇率：{round(ExRate,2)}）。"
    if n := GOLD.deal(bank_out, -xfer):
        return f"数量不足。\n——你还有{n}枚金币。"
    GOLD.deal(bank_in, int(receipt))
    return f"{group_out.nickname}向{group_in.nickname}转移{xfer} 金币\n汇率 {round(ExRate,2)}\n实际到账金额 {receipt}"


@plugin.handle(
    ["市场注册", "公司注册", "注册公司"],
    ["group_id", "to_me", "permission", "group_avatar"],
    rule=[Rule.to_me, Rule.group_admin],
)
async def _(event: Event):
    group_id = event.group_id
    group = manager.data.group(group_id)
    if group_avatar := event.group_avatar:
        group.avatar_url = group_avatar
    stock = group.stock
    if stock:
        return f"本群已在市场注册，注册名：{stock.name}"
    stock_name = event.single_arg()
    if not stock_name:
        return "请输入注册名"
    if check := item_name_rule(stock_name):
        return check
    if manager.group_library.get(stock_name) or manager.props_library.get(stock_name):
        return f"{stock_name} 已被注册"
    if (gold := group.bank[GOLD.id]) < company_public_gold:
        return f"本群金库金币（{gold}）小于{company_public_gold}，注册失败。"
    wealths = manager.group_wealths(group_id, GOLD.id)
    stock_value = sum(wealths)
    gini = gini_coef([x for x in wealths[:-1] if x >= gini_filter_gold])
    if gini > revolt_gini:
        return f"本群基尼系数（{round(gini,3)}）过高，注册失败。"
    level = group.level = (sum(ra.values()) if (ra := group.extra.get("revolution_achieve")) else 0) + 1
    group.bank[STD_GOLD.id] += gold * level
    group.bank[GOLD.id] = 0
    stock_value *= level
    stock = Stock(
        id=group_id,
        name=stock_name,
        value=stock_value,
        floating=stock_value,
        issuance=20000 * level,
        time=time.time(),
    )
    manager.group_library.set_item(group.id, {stock_name}, group)
    return f"{stock.name}发行成功，发行价格为{format_number(stock.value/ 20000)}金币"


@plugin.handle(
    ["公司重命名"],
    ["group_id", "to_me", "permission"],
    rule=[Rule.to_me, Rule.group_admin],
)
async def _(event: Event):
    group = manager.data.group(event.group_id)
    stock = group.stock
    if not stock:
        return "本群未在市场注册，不可重命名。"
    stock_name = event.single_arg()
    if not stock_name:
        return "请输入注册名"
    if check := item_name_rule(stock_name):
        return check
    if LICENSE.deal(group.bank, -1):
        return f"本群仓库缺少【{LICENSE.name}】"
    old_name = stock.name
    stock.name = stock_name
    manager.group_library.set_item(group.id, {stock_name}, group)
    return f"【{old_name}】已重命名为【{stock_name}】"


@plugin.handle(["购买", "发行购买"], ["user_id", "group_id", "nickname"])
async def _(event: Event):
    if not (args := event.args_parse()):
        return
    stock_name, buy, limit = args
    stock_group = manager.group_library.get(stock_name)
    if not stock_group or not (stock := stock_group.stock):
        return f"没有 {stock_name} 的注册信息"
    buy = min(stock_group.invest[stock.id], buy)
    if buy < 1:
        return "已售空，请等待结算。"
    if (n := stock_group.bank[GOLD.id]) < company_public_gold:
        return f"本群金币过少（{n}<{company_public_gold}），无法完成结算"
    stock_level = stock_group.level
    stock_value = sum(manager.group_wealths(stock.id, GOLD.id)) * stock_level + stock_group.bank[STD_GOLD.id]
    user, account = manager.account(event)
    group = manager.data.group(account.group_id)
    level = group.level
    account_STD_GOLD = account.bank[GOLD.id] * level
    my_STD_GOLD = user.bank[STD_GOLD.id] + account_STD_GOLD
    issuance = stock.issuance
    floating = stock.floating
    limit = limit or float("inf")
    value = 0.0
    _buy = 0
    for _ in range(buy):
        unit = max(floating, stock_value) / issuance
        if unit > limit:
            tip = f"价格超过限制（{limit}）。"
            break
        value += unit
        if value > my_STD_GOLD:
            value -= unit
            tip = f"你的金币不足（{my_STD_GOLD}）。"
            break
        floating += unit
        _buy += 1
    else:
        tip = "交易成功！"

    int_value = math.ceil(value)
    user.bank[STD_GOLD.id] -= int_value
    if (n := user.bank[STD_GOLD.id]) < 0:
        user.bank[STD_GOLD.id] = 0
        account_STD_GOLD += n
        account.bank[GOLD.id] = math.ceil(account_STD_GOLD / level)
        user.add_message(f"购买{_buy}份{stock_name}需要花费{int_value}枚标准金币，其中{-n}枚来自购买群账户，汇率（{level}）")
    user.invest[stock.id] += _buy
    stock_group.bank[GOLD.id] += math.ceil(value / stock_level)
    group.invest[stock.id] -= _buy
    stock.floating = floating
    stock.value = stock_value + int_value
    output = BytesIO()
    text_to_image(
        f"{stock.name}\n----" f"\n数量：{_buy}" f"\n单价：{round(value/_buy,2)}" f"\n总计：{int_value}" + endline(tip),
        width=440,
        bg_color="white",
    ).save(output, format="png")
    return output


@plugin.handle(["出售", "卖出", "结算"], ["user_id"])
async def _(event: Event):
    if not (args := event.args_parse()):
        return
    stock_name, n, quote = args
    user = manager.data.user(event.user_id)
    stock_group = manager.group_library.get(stock_name)
    if not stock_group or not (stock := stock_group.stock):
        return f"没有 {stock_name} 的注册信息"
    stock_name = stock_group.nickname
    my_stock = min(user.invest[stock.id], n)
    user_id = user.id
    exchange = stock.exchange
    if my_stock < 1:
        if user_id in exchange:
            del exchange[user_id]
            return "交易信息已注销。"
        else:
            return "交易信息无效。"
    if user_id in exchange:
        tip = "交易信息已修改。"
    else:
        tip = "交易信息发布成功！"
    exchange[user_id] = (n, quote or 0.0)
    output = BytesIO()
    text_to_image(
        f"{stock_name}\n----\n报价：{quote or '自动出售'}\n数量：{n}" + endline(tip),
        width=440,
        bg_color="white",
    ).save(output, format="png")
    return output


@plugin.handle(["市场信息"], ["user_id"])
async def _(event: Event):
    data = [(stock, group.invest[stock.id]) for group in manager.data.group_dict.values() if (stock := group.stock)]
    if not data:
        return "市场为空"
    data.sort(key=lambda x: x[0].value, reverse=True)
    return manager.info_card([invest_card(data)], event.user_id)


@plugin.handle(
    ["继承公司账户", "继承群账户"],
    ["user_id", "permission"],
    rule=Rule.superuser,
)
async def _(event: Event):
    args = event.args
    if len(args) != 3:
        return
    arrow = args[1]
    if arrow == "->":
        deceased = args[0]
        heir = args[2]
    elif arrow == "<-":
        heir = args[0]
        deceased = args[2]
    else:
        return "请输入:被继承群 -> 继承群"
    deceased_group = manager.group_library.get(deceased)
    if not deceased_group:
        return f"被继承群:{deceased} 不存在"
    heir_group = manager.group_library.get(heir)
    if not heir_group:
        return f"继承群:{heir} 不存在"
    if deceased_group is heir_group:
        return "无法继承自身"
    ExRate = deceased_group.level / heir_group.level
    # 继承群金库
    invest_group = Counter(deceased_group.invest)
    heir_group.invest = Counter(heir_group.invest) + invest_group
    bank_group = Counter({k: int(v * ExRate) if manager.props_library[k].domain == 1 else v for k, v in deceased_group.bank.items()})
    heir_group.bank = Counter(heir_group.bank) + bank_group
    # 继承群员账户
    all_bank_private = Counter()
    for deceased_user_id, deceased_account_id in deceased_group.accounts_map.items():
        deceased_account = manager.data.account_dict[deceased_account_id]
        bank = Counter({k: int(v * ExRate) for k, v in deceased_account.bank.items()})
        if deceased_user_id in heir_group.accounts_map:
            all_bank_private += bank
            heir_account_id = heir_group.accounts_map[deceased_user_id]
            heir_account = manager.data.account_dict[heir_account_id]
            heir_account.bank = Counter(heir_account.bank) + bank
        else:
            bank_group += bank
            heir_group.bank = Counter(heir_group.bank) + bank
    del manager.group_library[deceased_group.id]
    manager.data.cancel_group(deceased_group.id)
    info = []
    info.append(invest_card(manager.invest_data(invest_group), "群投资继承"))
    info.append(prop_card(manager.props_data(bank_group), "群金库继承"))
    info.append(prop_card(manager.props_data(all_bank_private), "个人总继承"))
    return manager.info_card(info, event.user_id)


@plugin.handle(["刷新市场"], ["permission"], rule=Rule.superuser)
@scheduler.scheduled_job("cron", minute="*/5", misfire_grace_time=120)
async def _():
    def stock_update(group: Group):
        stock = group.stock
        if not stock:
            return f"{group.id} 更新失败"
        level = group.level
        # 资产更新
        wealths = manager.group_wealths(group.id, GOLD.id)
        stock_value = stock.value = sum(wealths) * level + group.bank[STD_GOLD.id]
        floating = stock.floating
        if not floating or math.isnan(floating):
            stock.floating = float(stock_value)
            return f"{stock.name} 已初始化"
        # 股票价格变化：趋势性影响（正态分布），随机性影响（平均分布），向债务价值回归
        floating += floating * random.gauss(0, 0.03)
        floating += stock_value * random.uniform(-0.1, 0.1)
        floating += (stock_value - floating) * 0.05
        # 股票浮动收入
        group.bank[GOLD.id] = int(wealths[-1] * floating / stock.floating)
        # 结算交易市场上的股票
        issuance = stock.issuance
        std_value = 0
        now_time = time.time()
        clock = time.strftime("%H:%M", time.localtime(now_time))
        for user_id, exchange in stock.exchange.items():
            user = manager.data.user(user_id)
            n, quote = exchange
            value = 0.0
            settle = 0
            if quote:
                for _ in range(n):
                    unit = floating / issuance
                    if unit < quote:
                        break
                    value += quote
                    floating -= quote
                    settle += 1
            else:
                for _ in range(n):
                    unit = max(floating / issuance, 0.0)
                    value += unit
                    floating -= unit
                settle = n
            if settle == 0:
                continue
            elif settle < n:
                stock.exchange[user_id] = (n - settle, quote)
            else:
                stock.exchange[user_id] = (0, 0)
            user.invest[stock.id] -= settle
            group.invest[stock.id] += settle
            int_value = int(value)
            user.bank[STD_GOLD.id] += int_value
            user.message.append(
                f"【交易市场 {clock}】收入{int_value}标准金币。\n{stock.name}已出售{settle}/{n}，报价{quote or format_number(value/settle)}。"
            )
            std_value += value
        group.bank[GOLD.id] -= int(std_value / level)
        stock.exchange = {user_id: exchange for user_id, exchange in stock.exchange.items() if exchange[0] > 0}
        # 更新浮动价格
        stock.floating = floating
        # 记录价格历史
        if not (stock_record := group.extra.get("stock_record")):
            stock_record = [(0.0, 0.0) for _ in range(720)]
        stock_record.append((now_time, floating / issuance))
        stock_record = stock_record[-720:]
        group.extra["stock_record"] = stock_record
        return f"{stock.name} 更新成功！"

    groups = (group for group in manager.data.group_dict.values() if group.stock and group.stock.issuance)
    print("\n".join(stock_update(group) for group in groups))



import random
from pydantic import BaseModel

Event_list = list[tuple[int, "Event"]]
"""
[[概率值1, {事件}], [概率值2, {事件}], ......]

[[延迟回合数1, {事件}], [延迟回合数2, {事件}], ......]
"""


class Event(BaseModel):
    event_name: str = "未知事件"
    """事件名称"""
    only_key: int | None = None
    """唯一事件码，不为None则一场只触发一次"""
    describe: str = ""
    """事件描述"""
    target: int = -1
    """
    事件生效目标
        0: 自己
        1: 随机选择一个非自己的目标
        2: 全部
        3: 除自身外全部
        4: 全场随机一个目标
        5: 自己和一位其他目标
        6: 随机一侧赛道的马儿
        7: 自己两侧赛道的马儿
    """
    target_is_buff: str | None = None
    """筛选事件生效目标有buff名"""
    target_no_buff: str | None = None
    """筛选事件生效目标无buff名"""
    live: int = 0
    """复活事件：为1则目标复活"""
    move: int = 0
    """位移事件：目标立即进行相当于参数值的位移"""
    track_to_location: int | None = None
    """随机位置事件：有值则让目标移动到指定位置"""
    track_random_location: int | None = None
    """随机位置事件：为1则让目标随机位置（位置范围为可设定值，见setting.py）"""
    buff_time_add: int = 0
    """buff持续时间调整事件：目标所有buff增加/减少回合数"""
    del_buff: str | None = None
    """删除buff事件：下回合删除目标含特定buff_tag的所有buff"""
    track_exchange_location: int = 0
    """换位事件：值为1则与目标更换位置 （仅target为1,6时生效）"""
    random_event_once: Event_list = []
    """一次性随机事件"""
    die: int = 0
    """死亡：为1则目标死亡，此参数生成的buff默认持续到9999回合"""
    die_name: str = "死亡"
    """die的自定义名称"""
    away: int = 0
    """离开：为1则目标死亡，此参数生成的buff默认持续到9999回合"""
    away_name: str = "离开"
    """away的自定义名称"""
    rounds: int = 0
    """buff持续回合数"""
    name: str = "未命名buff"
    """buff名称，turn值>0时为必要值"""
    move_max: int = 0
    """该buff提供马儿每回合位移值区间的最大值"""
    move_min: int = 0
    """该buff提供马儿每回合位移值区间的最小值"""
    buffs: set = set()
    """buff组合，详见Buff类buffs字段文档"""
    random_event: Event_list = []
    """持续性随机事件，以buff形式存在"""
    delay_event: tuple[int, "Event"] | None = None
    """延迟事件（以当前事件的targets为发起人的事件）：前者为多少回合后，需>1"""
    delay_event_self: tuple[int, "Event"] | None = None
    """延迟事件（以当前事件发起人为发起人的事件）：前者为多少回合后，需>1"""
    another_event: "Event | None" = None
    """同步事件（以当前事件的targets为发起人的事件），执行此事件后立马执行该事件"""
    another_event_self: "Event | None" = None
    """同步事件（以当前事件发起人为发起人的事件），执行此事件后立马执行该事件"""
    add_horse: dict = {}
    """增加一匹马事件"""
    replace_horse: dict = {}
    """替换一匹马事件"""


class Buff(BaseModel):
    name: str
    """buff名称，turn值>0时为必要值"""
    round_start: int
    """buff开始回合数"""
    round_end: int
    """buff结束回合数"""
    move_min: int
    move_max: int
    buffs: set[str]
    """
    locate_lock: 止步，目标无法移动
    vertigo: 眩晕，目标无法移动，且不主动执行事件（暂定）
    hiding: 隐身：不显示目标移动距离及位置
    others: 自定义buff_tag，仅标识用buff_tag填写处，也可以填入常规buff_tag并正常生效
    """
    event_in_buff: Event_list

    def info(self):
        return f"buff名称：{self.name}\n回合：{self.round_start} - {self.round_end}\n标签：{self.buffs}"


class Horse:
    def __init__(self, horsename, uid, id, location=0, round=0):
        self.horse: str = horsename
        self.playeruid: str = uid
        self.player = id
        self.buff: list[Buff] = []
        self.delay_events: Event_list = []
        self.round = round
        self.location = location
        self.location_add = 0
        self.location_add_move = 0

    def info(self):
        return (
            f"马儿：{self.horse}\n"
            f"回合：{self.round}\n"
            f"位置：{self.location}\n"
            f"本回合移动：{self.location_add}\n"
            f"当前buff：\n{'\n'.join(buff.info() for  buff in self.buff)}\n"
        )

    def add_buff(
        self,
        buff_name: str,
        buffs: set[str],
        round_start: int,
        round_end: int,
        move_min: int = 0,
        move_max: int = 0,
        event_in_buff=[],
    ):
        """马儿buff增加"""
        if move_min > move_max:
            move_max = move_min
        buff = Buff(
            name=buff_name,
            round_start=round_start,
            round_end=round_end,
            move_min=move_min,
            move_max=move_max,
            buffs=buffs,
            event_in_buff=event_in_buff,
        )
        self.buff.append(buff)

    def del_buff(self, del_buff_key):
        """马儿指定buff移除"""
        self.buff = [buff for buff in self.buff if del_buff_key not in buff.buffs]

    def find_buff(self, find_buff_key):
        """马儿查找有无buff（查参数非名称）：(跳过计算回合数，只查有没有）"""
        return any(True for buff in self.buff if find_buff_key in buff.buffs)

    def buff_addtime(self, round_add):
        """马儿buff时间延长/减少"""
        for buff in self.buff:
            buff.round_end += round_add

    @property
    def is_stop(self) -> bool:
        """马儿是否止步"""
        return self.find_buff("locate_lock")

    @property
    def is_away(self) -> bool:
        """马儿是否已经离开"""
        return self.find_buff("away")

    @property
    def is_die(self) -> bool:
        """马儿是否已经死亡"""
        return self.find_buff("die")

    def location_move(self, move):
        """马儿移动计算（事件提供的本回合移动）"""
        self.location_add_move += move

    def location_to(self, move_to):
        """马儿移动至特定位置计算（事件提供移动）"""
        self.location_add_move = move_to - self.location

    def base_move(self, move_min: int, move_max: int) -> int:
        """马儿基础移动计算"""
        for buff in self.buff:
            move_min += buff.move_min
            move_max += buff.move_max
        base_move = random.randint(move_min, move_max)
        return base_move

    def move(self, base_move: int, track_length: int):
        """马儿移动计算"""
        self.location_add = base_move + self.location_add_move
        self.location += self.location_add
        self.location = max(0, self.location)
        self.location = min(track_length - 1, self.location)

    # =====赛马玩家战况显示：
    def display(self, track_length: int):
        if self.find_buff("hiding"):
            return "[+?]" + "." * track_length

        start = f"[{self.location_add}]" if self.location_add < 0 else f"[+{self.location_add}]"
        track = ["." for _ in range(track_length - 1)]
        track.insert(track_length - 1 - self.location, "".join(f"<{buff.name}>" for buff in self.buff) + self.horse)
        return start + "".join(track)



import time
import asyncio
from collections import Counter
from collections.abc import Coroutine, Callable, Sequence
from clovers.utils.tools import to_int
from clovers_leafgame.main import manager
from clovers_leafgame.item import Prop, GOLD
from clovers_leafgame.output import text_to_image, endline
from clovers_leafgame.core.clovers import Event
from clovers.core.config import config as clovers_config
from .config import Config

config_key = __package__
config_data = Config.model_validate(clovers_config.get(config_key, {}))
clovers_config[config_key] = config_data.model_dump()

default_bet = config_data.default_bet
timeout = config_data.timeout


class Session:
    """
    游戏场次信息
    """

    time: float
    group_id: str
    at: str | None = None
    p1_uid: str
    p1_nickname: str
    p2_uid: str = ""
    p2_nickname: str | None = None
    round = 1
    next: str
    win: str | None = None
    bet: tuple[Prop, int] | None = None
    data: dict = {}
    game: "Game"
    end_tips: str | None = None
    start_tips: ... = None

    def __init__(self, group_id: str, user_id: str, nickname: str, game: "Game"):
        self.time = time.time()
        self.group_id = group_id
        self.p1_uid = user_id
        self.p1_nickname = nickname
        self.next = user_id
        self.game = game

    def join(self, user_id: str, nickname: str):
        self.time = time.time()
        self.p2_uid = user_id
        self.p2_nickname = nickname

    def timeout(self):
        return timeout + self.time - time.time()

    def nextround(self):
        self.time = time.time()
        self.round += 1
        self.next = self.p1_uid if self.next == self.p2_uid else self.p2_uid

    def double_bet(self):
        if not self.bet:
            return
        prop, n = self.bet
        n += self.data["bet"]
        self.bet = (prop, min(n, self.data["bet_limit"]))

    def delay(self, t: float = 0):
        self.time = time.time() + t

    def create_check(self, user_id: str):
        p2_uid = self.p2_uid
        if not p2_uid:
            return
        p1_uid = self.p1_uid
        if p1_uid == user_id:
            return "你已发起了一场对决"
        if p2_uid == user_id:
            return "你正在进行一场对决"
        if p1_uid and p2_uid:
            return f"{self.p1_nickname} 与 {self.p2_nickname} 的对决还未结束，请等待比赛结束后再开始下一轮..."

    def action_check(self, user_id: str):
        if not self.p2_uid:
            if self.p1_uid == user_id:
                return "目前无人接受挑战哦"
            return "请先接受挑战"
        if self.p1_uid == user_id or self.p2_uid == user_id:
            if user_id == self.next:
                return
            return f"现在是{self.p1_nickname if self.next == self.p1_uid else self.p2_nickname}的回合"
        return f"{self.p1_nickname} v.s. {self.p2_nickname}\n正在进行中..."

    def create_info(self):
        if self.at:
            p2_nickname = self.p2_nickname or f"玩家{self.at[:4]}..."
            return f"{self.p1_nickname} 向 {p2_nickname} 发起挑战！\n请 {p2_nickname} 回复 接受挑战 or 拒绝挑战\n【{timeout}秒内有效】"
        else:
            return f"{self.p1_nickname} 发起挑战！\n回复 接受挑战 即可开始对局。\n【{timeout}秒内有效】"

    def settle(self):
        """
        游戏结束结算
            return:结算界面
        """
        group_id = self.group_id
        win = self.win if self.win else self.p1_uid if self.next == self.p2_uid else self.p2_uid
        if win == self.p1_uid:
            win_name = self.p1_nickname
            lose = self.p2_uid
            lose_name = self.p2_nickname
        else:
            win_name = self.p2_nickname
            lose = self.p1_uid
            lose_name = self.p1_nickname

        bet = self.bet
        if bet:
            tip = manager.transfer(*bet, lose, win, group_id)
            info = [text_to_image(tip + endline("结算"), autowrap=True)]
        else:
            info = []
        ranklist = manager.data.extra.setdefault("ranklist", {})
        win_rank = ranklist["win"] = Counter(ranklist.get("win", {}))
        win_rank[win] += 1
        lose_rank = ranklist["lose"] = Counter(ranklist.get("lose", {}))
        lose_rank[lose] += 1
        win_achieve = ranklist["win_achieve"] = Counter(ranklist.get("win_achieve", {}))
        win_achieve[win] += 1
        win_achieve[lose] = 0
        lose_achieve = ranklist["lose_achieve"] = Counter(ranklist.get("lose_achieve", {}))
        lose_achieve[win] = 0
        lose_achieve[lose] += 1
        card = (
            f"[pixel][20]◆胜者 {win_name}[nowrap]\n[pixel][460]◇败者 {lose_name}\n"
            f"[pixel][20]◆战绩 {win_rank[win]}:{lose_rank[win]}[nowrap]\n[pixel][460]◇战绩 {win_rank[lose]}:{lose_rank[lose]}\n"
            f"[pixel][20]◆连胜 {win_achieve[win]}[nowrap]\n[pixel][460]◇连败 {lose_achieve[lose]}"
        )
        info.insert(0, text_to_image(card + endline("对战")))
        result = [f"这场对决是 {win_name} 胜利了", manager.info_card(info, win)]
        if self.end_tips:
            result.append(self.end_tips)
        return result

    def end(self, result=None):
        self.time = -1
        settle = self.settle()

        async def output():
            if result:
                yield result
                await asyncio.sleep(1)
            for x in settle:
                yield x
                await asyncio.sleep(1)

        return output()


class Game:
    def __init__(self, name: str, action_tip: str) -> None:
        self.name = name
        self.action_tip = action_tip

    @staticmethod
    def args_parse(args: Sequence[str]) -> tuple[str, int, str]:
        match len(args):
            case 0:
                return "", 0, ""
            case 1:
                arg = args[0]
                return arg, 0, ""
            case 2:
                name, n = args
                return name, to_int(n) or 0, ""
            case _:
                arg, n, name = args[:3]
                if num := to_int(n):
                    n = num
                elif num := to_int(name):
                    name = n
                    n = num
                else:
                    n = 0
                return arg, n, name

    @staticmethod
    def session_check(place: dict[str, Session], group_id: str):
        if not (session := place.get(group_id)):
            return
        if session.timeout() < 0:
            del place[group_id]
            return
        return session

    def create(self, place: dict[str, Session]):
        def decorator(func: Callable[[Session, str], Coroutine]):
            async def wrapper(event: Event):
                user_id = event.user_id
                group_id = event.group_id
                if (session := self.session_check(place, group_id)) and (tip := session.create_check(user_id)):
                    return tip
                arg, n, prop_name = self.args_parse(event.args)
                prop = manager.props_library.get(prop_name, GOLD)
                user, account = manager.locate_account(user_id, group_id)
                user.connect = group_id
                bank = prop.locate_bank(user, account)
                if n < 0:
                    n = default_bet
                if n > bank[prop.id]:
                    return f"你没有足够的{prop.name}支撑这场对决({bank[prop.id]})。"
                session = place[group_id] = Session(group_id, user_id, account.name or user.name, game=self)
                if event.at:
                    session.at = event.at[0]
                    session.p2_nickname = manager.locate_account(session.at, group_id)[1].name
                if n:
                    session.bet = (prop, n)
                return await func(session, arg)

            return wrapper

        return decorator

    def action(self, place: dict[str, Session]):
        def decorator(func: Callable[[Event, Session], Coroutine]):
            async def wrapper(event: Event):
                group_id = event.group_id or manager.data.user(event.user_id).connect
                session = place.get(group_id)
                if not session or session.game.name != self.name or session.time == -1:
                    return
                user_id = event.user_id
                if tip := session.action_check(user_id):
                    return tip
                return await func(event, session)

            return wrapper

        return decorator



from pydantic import BaseModel


class Config(BaseModel):
    # 主路径
    main_path: str = "./LeafGames"
    # 默认显示字体
    fontname: str = "simsun"
    # 默认备用字体
    fallback_fonts: list[str] = [
        "arial",
        "tahoma",
        "msyh",
        "seguiemj",
    ]



"""+++++++++++++++++
————————————————————
    ᕱ⑅ᕱ。 ᴍᴏʀɴɪɴɢ
   (｡•ᴗ-)_
————————————————————
+++++++++++++++++"""

from datetime import datetime, timedelta
from pathlib import Path
import shutil
from collections import Counter
from clovers.utils.linecard import info_splicing, ImageList
from clovers.utils.library import Library
from .core.clovers import Event
from .core.data import Account, Group, Account, DataBase
from .item import Prop, props_library, marking_library, VIP_CARD


class Manager:
    data: DataBase
    main_path: Path

    def __init__(self, main_path: str | Path) -> None:
        self.main_path = Path(main_path)
        self.DATA_PATH = self.main_path / "russian_data.json"
        self.BG_PATH = self.main_path / "BG_image"
        self.BG_PATH.mkdir(exist_ok=True, parents=True)
        self.backup_path = self.main_path / "backup"
        self.backup_path.mkdir(exist_ok=True, parents=True)
        self.props_library = props_library
        self.marking_library = marking_library
        self.group_library: Library[str, Group] = Library()
        self.load()

    def save(self):
        with open(self.DATA_PATH, "w", encoding="utf8") as f:
            f.write(self.data.model_dump_json(indent=4))

    def load(self):
        self.data = DataBase.load(self.DATA_PATH)
        for group in self.data.group_dict.values():
            if (stock := group.stock) and (stock_name := stock.name):
                self.group_library.set_item(group.id, {stock_name}, group)
            else:
                self.group_library[group.id] = group

    def backup(self):
        date_today, now_time = datetime.now().strftime("%Y-%m-%d %H-%M-%S").split()
        backup_today = self.backup_path / date_today
        if not backup_today.exists():
            backup_today.mkdir(mode=755)
        file = backup_today / f"russian_data {now_time}.json"
        file.write_text(self.data.model_dump_json(indent=4), "utf8")

    def clean_backup(self, delta: int | float):
        folders = [f for f in self.backup_path.iterdir() if f.is_dir()]
        info = []
        timestamp = datetime.now().timestamp()
        for folder in folders:
            if timestamp - folder.stat().st_birthtime > delta:
                shutil.rmtree(folder)
                info.append(f"备份 {folder} 已删除！")
        return "\n".join(info)

    def info_card(self, info: ImageList, user_id: str, BG_type=None):
        extra = self.data.user(user_id).extra
        BG_type = BG_type or extra.get("BG_type", "#FFFFFF99")
        BG_PATH = self.BG_PATH / f"{user_id}.png"
        if not BG_PATH.exists():
            BG_PATH = self.BG_PATH / "default.png"
        return info_splicing(info, BG_PATH, spacing=10, BG_type=BG_type)

    def new_account(self, user_id: str, group_id: str, **kwargs):
        account = Account(user_id=user_id, group_id=group_id, sign_in=datetime.today() - timedelta(days=1), **kwargs)
        self.data.register(account)
        return account

    def locate_account(self, user_id: str, group_id: str):
        user = self.data.user(user_id)
        account_id = user.accounts_map.get(group_id)
        if not (account_id and (account := self.data.account_dict.get(account_id))):
            account = self.new_account(user_id, group_id)
        return user, account

    def account(self, event: Event):
        """
        定位账户
        """
        user_id = event.user_id
        user = self.data.user(user_id)
        group_id = event.group_id or user.connect
        account_id = user.accounts_map.get(group_id)
        if not (account_id and (account := self.data.account_dict.get(account_id))):
            account = self.new_account(user_id, group_id)
        nickname = event.nickname
        if event.nickname:
            account.name = nickname
            if not user.name or event.is_private():
                user.name = event.nickname
        return user, account

    def transfer(
        self,
        prop: Prop,
        unsettled: int,
        sender_id: str,
        receiver_id: str,
        group_id: str,
    ):
        sender, sender_account = self.locate_account(sender_id, group_id)
        receiver, receiver_account = self.locate_account(receiver_id, group_id)
        if prop.domain == 1:
            sender_bank = sender_account.bank
            receiver_bank = receiver_account.bank
        else:
            sender_bank = sender.bank
            receiver_bank = receiver.bank
        sender_name = sender_account.name or sender.name or sender.id
        if n := prop.deal(sender_bank, -unsettled):
            return f"数量不足。\n——{sender_name}还有{n}个{prop.name}。"
        receiver_name = receiver_account.name or receiver.name or receiver.id
        if sender_account.bank[VIP_CARD.id]:
            tax = 0
            tip = f"『{VIP_CARD.name}』免手续费"
        else:
            tax = int(unsettled * 0.02)
            tip = f"扣除2%手续费：{tax}，实际到账{prop.name}数{unsettled - tax}"
        prop.deal(receiver_bank, unsettled - tax)
        prop.deal(self.data.group(group_id).bank, tax)
        return f"{sender_name} 向 {receiver_name} 赠送了{unsettled}个{prop.name}\n{tip}"

    def group_wealths(self, group_name: str, prop_id: str) -> list[int]:
        """
        群内总资产
        """
        group = self.group_library.get(group_name)
        if not group:
            return []
        wealths = [self.data.account_dict[account_id].bank[prop_id] for account_id in group.accounts_map.values()]
        wealths.append(group.bank[prop_id])
        return wealths

    def stock_value(self, invest: Counter[str]):
        value = 0.0
        for group_id, n in invest.items():
            group = self.data.group_dict.get(group_id)
            if group and (stock := group.stock) and stock.name and stock.issuance > 0:
                value += stock.value * n / stock.issuance
            else:
                invest[group_id] = 0
        return int(value)

    def invest_data(self, bank: Counter[str]):
        def get_stock(group_id: str):
            group = self.group_library.get(group_id)
            if not group:
                return
            return group.stock

        return [(stock, n) for group_id, n in bank.items() if n != 0 and (stock := get_stock(group_id))]

    def props_data(self, bank: Counter[str]):
        return [(prop, n) for prop_id, n in bank.items() if n != 0 and (prop := self.props_library.get(prop_id))]



import random
import asyncio
import time
from datetime import datetime, timedelta
from collections import Counter
from clovers.core.plugin import Plugin
from clovers_leafgame.core.clovers import Event, Rule
from clovers_leafgame.main import plugin, manager
from clovers_leafgame.item import Prop, GOLD, STD_GOLD
from .core import usage, gacha, AIR_PACK, RED_PACKET
from clovers_leafgame.output import prop_card, bank_card
from .output import report_card

from clovers.core.config import config as clovers_config
from .config import Config

config_key = __package__
config_data = Config.model_validate(clovers_config.get(config_key, {}))
clovers_config[config_key] = config_data.model_dump()

gacha_gold = config_data.gacha_gold
packet_gold = config_data.packet_gold
luckey_min, luckey_max = config_data.luckey_coin
ticket_price = gacha_gold * 50


@plugin.handle(
    r"^(.+)连抽?卡?|单抽",
    ["user_id", "group_id", "nickname", "to_me"],
    rule=Rule.to_me,
)
async def _(event: Event):
    count = event.args_to_int()
    if not count:
        return
    count = 200 if count > 200 else 1 if count < 1 else count
    gold = count * gacha_gold
    user, account = manager.account(event)
    if n := GOLD.deal(account.bank, -gold):
        return f"{count}连抽卡需要{gold}金币，你的金币：{n}。"

    prop_data: list[list[tuple[Prop, int]]] = [[], [], []]
    report_data = {"prop_star": 0, "prop_n": 0, "air_star": 0, "air_n": 0}
    for prop_id, n in Counter(gacha() for _ in range(count)).items():
        prop = manager.props_library[prop_id]
        prop_data[prop.domain].append((prop, n))
        if prop.domain == 0:
            star_key = "air_star"
            n_key = "air_n"
        else:
            star_key = "prop_star"
            n_key = "prop_n"
            prop.deal(prop.locate_bank(user, account), n)
        report_data[star_key] += prop.rare * n
        report_data[n_key] += n
    if count < 10:
        return "你获得了" + "\n".join(f"({prop.rare}☆){prop.name}:{n}个" for seg in prop_data for prop, n in seg)
    else:
        info = [report_card(account.name, **report_data)]
        if report_data["prop_n"] == 0:
            AIR_PACK.deal(user.bank, 1)
            RED_PACKET.deal(account.bank, 10)
            GOLD.deal(account.bank, gold)
            info.append(prop_card([(AIR_PACK, 1), (GOLD, gold), (RED_PACKET, 10)], f"本次抽卡已免费"))
        if data := prop_data[2]:
            info.append(prop_card(data, "全局道具"))
        if data := prop_data[1]:
            info.append(prop_card(data, "群内道具"))
        if data := prop_data[0]:
            info.append(prop_card(data, "未获取"))

    return manager.info_card(info, user.id)


@usage("金币", ["user_id", "group_id", "nickname"])
async def _(prop: Prop, event: Event, count: int, extra: str):
    if n := prop.deal(prop.locate_bank(*manager.account(event)), -count):
        return f"使用失败，你还有{n}枚金币。"
    return f"你使用了{count}枚金币。"


@usage("临时维护凭证", ["user_id", "group_id", "nickname"])
async def _(prop: Prop, event: Event, count: int, extra: str):
    bank = prop.locate_bank(*manager.account(event))
    if prop.deal(bank, -1):
        return f"使用失败，你未持有{prop.name}"
    try:
        exec(extra.strip())
        return f"执行成功！"
    except Exception as e:
        prop.deal(bank, 1)
        return str(e)


@usage("测试金库", ["user_id", "group_id", "nickname"])
async def _(prop: Prop, event: Event, count: int, extra: str):
    bank = prop.locate_bank(*manager.account(event))
    if prop.deal(bank, -1):
        return f"使用失败，你未持有{prop.name}"
    return "你获得了10亿金币，100万钻石。祝你好运！"


@usage("空气礼包", ["user_id", "group_id", "nickname"])
async def _(prop: Prop, event: Event, count: int, extra: str):
    user, account = manager.account(event)
    bank = prop.locate_bank(user, account)
    if n := prop.deal(bank, -count):
        return f"使用失败，你没有足够的{prop.name}（{n}）"
    data = [(prop, count) for prop in manager.props_library.values() if prop.domain == 0]
    bank += {prop.id: n for prop, n in data}
    return ["你获得了", manager.info_card(bank_card(data), user.id)]


@usage("随机红包", ["user_id", "group_id", "nickname"])
async def _(prop: Prop, event: Event, count: int, extra: str):
    user, account = manager.account(event)
    bank = prop.locate_bank(user, account)
    if n := prop.deal(bank, -count):
        return f"使用失败，你没有足够的{prop.name}（{n}）"
    gold = random.randint(*packet_gold) * count
    GOLD.deal(account.bank, gold)
    return f"你获得了{gold}金币。祝你好运~"


@usage("重开券", ["user_id", "group_id", "nickname"])
async def _(prop: Prop, event: Event, count: int, extra: str):
    user, account = manager.account(event)
    bank = prop.locate_bank(user, account)
    if prop.deal(bank, -1):
        return f"使用失败，你没有足够的{prop.name}"
    if (n := account.bank[GOLD.id]) < 0:
        std_n = n * manager.data.group(account.group_id).level
        user.bank[STD_GOLD.id] += std_n
        user.add_message(f"【{prop.name}】你在群内的欠款（{-std_n}枚标准金币）已转移到个人账户")
    data = manager.props_data(bank)
    manager.data.cancel_account(account.id)
    return ["你在本群的账户已重置，祝你好运~", manager.info_card([prop_card(data, "账户已重置")], user.id)]


@usage("幸运硬币", ["user_id", "group_id", "nickname", "Bot_Nickname"])
async def _(prop: Prop, event: Event, count: int, extra: str):
    user, account = manager.account(event)
    count = min(count, account.bank[GOLD.id])
    if count < luckey_min:
        return f"使用失败，{prop.name}最小赌注为{luckey_min}金币"
    if count > luckey_max:
        return f"使用失败，{prop.name}最大赌注为{luckey_max}金币"
    bank = prop.locate_bank(user, account)
    if prop.deal(bank, -1):
        return f"使用失败，你没有足够的{prop.name}"
    GOLD.deal(account.bank, -count)
    if random.randint(0, 1) == 0:
        GOLD.deal(account.bank, count * 2)
        return f"你获得了{count}金币"
    else:
        RED_PACKET.deal(RED_PACKET.locate_bank(user, account), 1)
        return f"你失去了{count}金币。\n{event.event.kwargs['Bot_Nickname']}送你1个『随机红包』，祝你好运~"


@usage("超级幸运硬币", ["user_id", "group_id", "nickname", "Bot_Nickname"])
async def _(prop: Prop, event: Event, count: int, extra: str):
    user, account = manager.account(event)
    bank = prop.locate_bank(user, account)
    if prop.deal(bank, -1):
        return f"使用失败，你没有足够的{prop.name}"
    gold = account.bank[GOLD.id]
    if random.randint(0, 1) == 0:
        account.bank[GOLD.id] *= 2
        return f"你获得了{gold}金币"
    else:
        account.bank[GOLD.id] = 0
        RED_PACKET.deal(RED_PACKET.locate_bank(user, account), 1)
        return f"你失去了{gold}金币。\n{event.event.kwargs['Bot_Nickname']}送你1个『随机红包』，祝你好运~"


@usage("道具兑换券", ["user_id", "group_id", "nickname"])
async def _(prop: Prop, event: Event, count: int, extra: str):
    if extra:
        prop_name = extra.strip()
        target_prop = manager.props_library.get(prop_name)
        if not target_prop:
            return f"不存在道具【{prop_name}】"
        if target_prop.rare < 3:
            return f"无法兑换【{target_prop.name}】"
    else:
        target_prop = prop
    user, account = manager.account(event)
    bank = prop.locate_bank(user, account)
    # 购买道具兑换券，价格 50抽
    if count > bank[prop.id]:
        gold = ticket_price * (count - bank[prop.id])
        if n := GOLD.deal(account.bank, -gold):
            return f"金币不足。你还有{n}枚金币。（需要：{gold}）"
        bank[prop.id] = 0
    else:
        gold = 0
        bank[prop.id] -= count

    target_bank = target_prop.locate_bank(user, account)
    target_bank[target_prop.id] += count
    return f"你获得了{count}个【{target_prop.name}】！（使用金币：{gold}）"


@usage("绯红迷雾之书", ["user_id", "group_id", "nickname"])
async def _(prop: Prop, event: Event, count: int, extra: str):
    user_id = event.user_id
    bank = manager.data.user(user_id).bank
    if bank[prop.id] < 1:
        return f"使用失败，你未持有{prop.name}"
    group_id = event.group_id
    folders = {f.name: f for f in manager.backup_path.iterdir() if f.is_dir()}
    tip = "请输入你要回档的日期:\n" + "\n".join(folders.keys())
    key = f"{user_id} {group_id}"

    @plugin.temp_handle(key, extra_args=["user_id", "group_id"], rule=Rule.locate(user_id, group_id))
    async def _(event: Event, finish):
        date = event.raw_command
        folder = folders.get(date)
        if not folder:
            return tip
        files = {f.stem.split()[1].replace("-", ":"): f for f in folder.iterdir() if f.is_file()}
        tip2 = "请输入你要回档的时间:\n" + "\n".join(files.keys())
        finish()

        @plugin.temp_handle(key, extra_args=["user_id", "group_id"], rule=Rule.locate(user_id, group_id))
        async def _(event: Event, finish):
            clock = event.raw_command
            file = files.get(clock)
            if not file:
                return tip2
            manager.data.cancel_user(user_id)
            old_data = manager.data.load(file)
            user = manager.data.user_dict[user_id] = old_data.user(user_id)
            user.bank[prop.id] = bank[prop.id] - 1
            for account_id in user.accounts_map.values():
                manager.data.register(old_data.account_dict[account_id])
            finish()
            return f"你已经回档到{date} {clock}"

        return tip2

    return tip


@usage("恶魔轮盘", ["user_id", "group_id", "nickname", "Bot_Nickname"])
async def _(prop: Prop, event: Event, count: int, extra: str):
    user_id = event.user_id
    group_id = event.group_id
    user = manager.data.user(user_id)
    if user.bank[prop.id] < 1:
        return f"使用失败，你没有足够的{prop.name}"
    # 增加一步验证，判断玩家是否一星期内失败过。
    death_cd: dict[str, float] = manager.data.extra.setdefault("death_cd", {})
    if user_id in death_cd:
        death_date = datetime.fromtimestamp(death_cd[user_id])
        revival_date = (death_date + timedelta(days=7 - death_date.weekday())).replace(hour=0, minute=0, second=0, microsecond=0)
        if datetime.now() < revival_date:
            return f"您需要在{revival_date.strftime('%Y年%m月%d日')}后才能使用此道具"
        del death_cd[user_id]

    @plugin.temp_handle(f"{user_id} {group_id}", extra_args=["user_id", "group_id"], timeout=60, rule=Rule.locate(user_id, group_id))
    async def _(event: Event, finish: Plugin.Finish):
        finish()
        command = event.raw_command
        if command == "取消":
            return f"你取消了恶魔轮盘"
        elif command == "开枪":

            async def result():
                bullet_lst = [0, 0, 0, 0, 0, 0]
                for i in random.sample([0, 1, 2, 3, 4, 5], random.randint(0, 6)):
                    bullet_lst[i] = 1
                index = random.randint(0, 5)
                yield f"子弹列表{" ".join(str(x) for x in bullet_lst)}，你中了第{index+1}发子弹。"
                await asyncio.sleep(0.5)
                if bullet_lst[index] == 1:
                    death_cd[user_id] = time.time()
                    manager.data.cancel_user(user_id)
                    yield "砰！一团火从枪口喷出，你从这个世界上消失了。"
                else:
                    yield "咔！你活了下来..."

                    def rate_times_bank(bank: Counter, rate: int):
                        for k in bank.keys():
                            bank[k] *= rate

                    counter = Counter()
                    rate_times_bank(user.bank, 10)
                    user.bank[prop.id] = 1
                    counter += user.bank
                    for account_id in user.accounts_map.values():
                        account = manager.data.account_dict[account_id]
                        rate_times_bank(account.bank, 10)
                        counter += account.bank
                    user.bank[STD_GOLD.id] += manager.stock_value(user.invest) * 10

                    yield ["这是你获得的道具", manager.info_card([prop_card(manager.props_data(counter))], user_id)]

            return result()

    return "你手中的左轮枪已经装好了子弹，请开枪，或者取消。"



"""
小游戏框架基础道具实例
    AIR:空气
    GOLD:金币
    VIP_CARD:钻石会员卡
    LICENSE:设置许可证
"""

import os
import json
from pathlib import Path
from .core.data import Prop
from clovers.utils.library import Library


def item_name_rule(item_name: str):
    if " " in item_name or "\n" in item_name:
        return "名称不能含有空格或回车"
    count = 0
    for x in item_name:
        if ord(x) < 0x200:
            count += 1
        else:
            count += 2
    if count > 24:
        return f"名称不能超过24字符"
    try:
        int(item_name)
        return f"名称不能是数字"
    except:
        return None


props_library_file = Path(os.path.join(os.path.dirname(__file__), "props_library.json"))
props_library: Library[str, Prop] = Library()
with open(props_library_file, "r", encoding="utf8") as f:
    for k, v in json.load(f).items():
        prop = Prop(k, **v)
        props_library.set_item(prop.id, [prop.name], prop)

AIR = props_library["空气"]
GOLD = props_library["金币"]
STD_GOLD = props_library["标准金币"]
VIP_CARD = props_library["钻石会员卡"]
LICENSE = props_library["设置许可证"]
CLOVERS_MARKING = props_library["四叶草标记"]
REVOLUTION_MARKING = props_library["路灯挂件标记"]
DEBUG_MARKING = props_library["Debug奖章"]
PROP_FOR_TEST = props_library["测试金库"]

marking_library: Library[str, Prop] = Library()
marking_library.set_item(PROP_FOR_TEST.id, [PROP_FOR_TEST.name], PROP_FOR_TEST)
marking_library.set_item(AIR.id, [AIR.name], AIR)



from pydantic import BaseModel


class Config(BaseModel):
    # 玩家人数范围
    range_of_player_numbers: tuple[int, int] = (2, 8)
    # 跑道长度
    setting_track_length: int = 30
    # 随机位置事件，能够随机到的跑道范围
    random_move_range: tuple[float, float] = (0.0, 0.8)
    # 每回合基础移动范围
    base_move_range: tuple[int, int] = (1, 3)
    # 事件概率
    event_randvalue: int = 450



import random
import os
import json
from pathlib import Path
from collections.abc import Callable, Coroutine
from clovers_leafgame.core.clovers import Event
from clovers_leafgame.item import Prop, AIR
from clovers_leafgame.main import plugin, manager

library_file = Path(os.path.join(os.path.dirname(__file__), "./props_library.json"))
with open(library_file, "r", encoding="utf8") as f:
    for k, v in json.load(f).items():
        prop = Prop(k, **v)
        manager.props_library.set_item(prop.id, [prop.name], prop)

pool = {
    rare: [manager.props_library[name].id for name in name_list]
    for rare, name_list in {
        3: ["优质空气", "四叶草标记", "挑战徽章", "设置许可证", "初级元素"],
        4: ["高级空气", "钻石会员卡"],
        5: ["特级空气", "进口空气", "幸运硬币"],
        6: ["纯净空气", "钻石", "道具兑换券", "超级幸运硬币", "重开券"],
    }.items()
}
AIR_PACK = manager.props_library["空气礼包"]
RED_PACKET = manager.props_library["随机红包"]
DIAMOND = manager.props_library["钻石"]


def gacha() -> str:
    """
    随机获取道具。
        return: object_code
    """
    rand = random.uniform(0.0, 1.0)
    prob_list = (0.3, 0.1, 0.1, 0.02)
    rare = 3
    for prob in prob_list:
        rand -= prob
        if rand <= 0:
            break
        rare += 1
    if rare_pool := pool.get(rare):
        return random.choice(rare_pool)
    return AIR.id


def usage(prop_name: str, extra_args):
    def decorator(func: Callable[..., Coroutine]):
        prop = manager.props_library.get(prop_name)
        assert prop is not None, f"不存在道具{prop_name}，无法注册使用方法。"

        @plugin.handle(f"使用(道具)?\\s*{prop_name}\\s*(\\d*)(.*)", extra_args)
        async def _(event: Event):
            _, count, extra = event.args
            return await func(prop, event, int(count) if count else 1, extra)

    return decorator



from pydantic import BaseModel


class Config(BaseModel):
    # 抽卡所需金币
    gacha_gold: int = 50
    # 礼包金币范围
    packet_gold: tuple[int, int] = (200, 2000)
    # 幸运硬币赌注范围
    luckey_coin: tuple[int, int] = (2000, 100000)



from datetime import datetime
from collections import Counter
from clovers_apscheduler import scheduler
from clovers_leafgame.core.clovers import Event, Rule
from clovers_leafgame.main import plugin, manager


def verification():
    """
    数据校验
    """
    user_dict = manager.data.user_dict
    group_dict = manager.data.group_dict
    account_dict = manager.data.account_dict
    props_library = manager.props_library
    stock_check = Counter()
    # 检查 user_dict
    for user_id, user in user_dict.items():
        user.id = user_id
        # 清理未持有的道具
        user.bank = Counter({k: v for k, v in user.bank.items() if v > 0 and k in props_library})
        # 删除无效及未持有的股票
        invest = user.invest = Counter({k: v for k, v in user.invest.items() if k in group_dict and v})
        # 股票数检查
        stock_check += Counter(invest)
        for group_id, accounts_id in user.accounts_map.items():
            account = account_dict[accounts_id]
            account.user_id = user_id
            account.group_id = group_id
            # 清理未持有的道具
            account.bank = Counter({k: v for k, v in account.bank.items() if v > 0 and k in props_library})
            group_dict[group_id].accounts_map[user_id] = accounts_id
    # 检查 group_dict
    for group_id, group in group_dict.items():
        # 清理未持有的道具
        group.bank = Counter({k: v for k, v in group.bank.items() if v > 0 and k in props_library})
        # 删除无效及未持有的股票
        group.invest = Counter({k: v for k, v in group.invest.items() if k in group_dict and v})
        # 修正公司等级
        group.level = sum(group.extra.setdefault("revolution_achieve", {}).values()) + 1
        stock = group.stock
        if not stock:
            continue
        # 修正股票库存
        stock.id = group_id
        issuance = 20000 * group.level
        stock.issuance = issuance
        group.invest[group_id] = issuance - stock_check[group_id]


# 数据验证
@plugin.handle(["数据验证"], ["permission"], rule=Rule.superuser)
async def _(event: Event):
    verification()
    print("数据已验证")


@plugin.handle(["保存游戏"], ["permission"], rule=Rule.superuser)
@scheduler.scheduled_job("cron", minute="*/10", misfire_grace_time=120)
async def _(*arg, **kwargs):
    manager.save()
    print("游戏数据已保存！")


@plugin.handle(["刷新每日"], ["permission"], rule=Rule.superuser)
@scheduler.scheduled_job("cron", hour=0, misfire_grace_time=120)
async def _(*arg, **kwargs):
    revolution_today = datetime.today().weekday() in {4, 5, 6}
    for user in manager.data.user_dict.values():
        bank = {k: (v - 1) if prop.flow == 0 else v for k, v in user.bank.items() if (prop := manager.props_library.get(k))}
        user.bank = Counter(bank)
    for account in manager.data.account_dict.values():
        # 周末刷新重置签到
        account.extra["revolution"] = revolution_today
        # 群内道具有效期 - 1天
        bank = {k: (v - 1) if prop.flow == 0 else v for k, v in account.bank.items() if (prop := manager.props_library.get(k))}
        account.bank = Counter(bank)
    verification()
    print("每日签到已刷新")


# 数据备份
@plugin.handle(["数据备份"], ["permission"], rule=Rule.superuser)
@scheduler.scheduled_job("cron", hour="*/4", misfire_grace_time=120)
async def _(*arg, **kwargs):
    manager.backup()
    print(manager.clean_backup(604800))



import random
from pathlib import Path
from datetime import datetime
from PIL import ImageColor
from collections import Counter
from clovers.utils.tools import download_url, format_number
from clovers_leafgame.core.clovers import Event, Rule
from clovers_leafgame.main import plugin, manager
from clovers_leafgame.item import (
    GOLD,
    STD_GOLD,
    LICENSE,
    CLOVERS_MARKING,
    REVOLUTION_MARKING,
    DEBUG_MARKING,
)
from clovers_leafgame.output import (
    text_to_image,
    endline,
    candlestick,
    bank_card,
    prop_card,
    invest_card,
    avatar_card,
    dist_card,
)
from clovers.core.config import config as clovers_config
from .config import Config


config_key = __package__
config_data = Config.model_validate(clovers_config.get(config_key, {}))
clovers_config[config_key] = config_data.model_dump()

sign_gold = config_data.sign_gold
clovers_marking = config_data.clovers_marking
revolution_marking = config_data.revolution_marking
debug_marking = config_data.debug_marking


@plugin.handle(["设置背景"], ["user_id", "to_me", "image_list"], rule=Rule.to_me)
async def _(event: Event):
    user_id = event.user_id
    user = manager.data.user(user_id)
    if LICENSE.deal(user.bank, -1):
        return f"你未持有【{LICENSE.name}】"
    log = []
    BG_type = event.single_arg()
    if BG_type:
        if BG_type in {"高斯模糊", "模糊"}:
            user.extra["BG_type"] = "GAUSS"
        elif BG_type in {"无", "透明"}:
            user.extra["BG_type"] = "NONE"
        elif BG_type == "默认":
            if "BG_type" in user.extra:
                del user.extra["BG_type"]
        else:
            try:
                ImageColor.getcolor(BG_type, "RGB")
                user.extra["BG_type"] = BG_type
            except ValueError:
                BG_type = "ValueError"
                pass
        log.append(f"背景蒙版类型设置为：{BG_type}")

    if url_list := event.image_list:
        image = await download_url(url_list[0])
        if not image:
            log.append("图片下载失败")
        else:
            with open(manager.BG_PATH / f"{user_id}.png", "wb") as f:
                f.write(image)
            log.append("图片下载成功")
    if log:
        return "\n".join(log)


@plugin.handle(["删除背景"], ["user_id", "to_me"], rule=Rule.to_me)
async def _(event: Event):
    Path.unlink(manager.BG_PATH / f"{event.user_id}.png", True)
    return "背景图片删除成功！"


@plugin.handle(["金币签到", "轮盘签到"], ["user_id", "group_id", "nickname", "avatar"])
async def _(event: Event):
    user, account = manager.account(event)
    if avatar := event.avatar:
        user.avatar_url = avatar
    today = datetime.today()
    if account.sign_in and (today - account.sign_in).days == 0:
        return "你已经签过到了哦"
    n = random.randint(*sign_gold)
    account.sign_in = today
    GOLD.deal(account.bank, n)
    return random.choice(["祝你好运~", "可别花光了哦~"]) + f"\n你获得了 {n} 金币"


@plugin.handle(["发红包"], ["user_id", "group_id", "at", "permission"], rule=Rule.at)
async def _(event: Event):
    unsettled = event.args_to_int()
    sender_id = event.user_id
    receiver_id = event.at[0]
    if unsettled < 0:
        if event.permission < 2:
            return "你输入了负数，请不要这样做。"
        sender_id, receiver_id = receiver_id, sender_id
        unsettled = -unsettled
    return manager.transfer(GOLD, unsettled, sender_id, receiver_id, event.group_id)


@plugin.handle(["送道具"], ["user_id", "group_id", "at", "permission"], rule=Rule.at)
async def _(event: Event):
    if not (args := event.args_parse()):
        return
    prop_name, unsettled = args[:2]
    prop = manager.props_library.get(prop_name)
    if not prop:
        return f"没有【{prop_name}】这种道具。"
    sender_id = event.user_id
    receiver_id = event.at[0]
    if unsettled < 0:
        if event.permission < 2:
            return "你输入了负数，请不要这样做。"
        sender_id, receiver_id = receiver_id, sender_id
        unsettled = -unsettled
    return manager.transfer(prop, unsettled, sender_id, receiver_id, event.group_id)


@plugin.handle(r"(.+)查询$", ["user_id", "group_id"])
async def _(event: Event):
    prop = manager.props_library.get(event.args[0])
    if not prop:
        return
    user_id = event.user_id
    user = manager.data.user(user_id)
    if prop.domain != 1:
        return f"你还有 {user.bank[prop.id]} 个{prop.name}"
    if event.is_private():
        info = []
        for group_id, account_id in user.accounts_map.items():
            account = manager.data.account_dict[account_id]
            info.append(f"【{manager.data.group(group_id).nickname}】{prop.name} {account.bank[prop.id]}个")
        if info:
            return "你的账户:\n" + "\n".join(info)
        else:
            return "你的账户是空的"
    group_id = event.group_id
    account_id = user.accounts_map.get(group_id)
    if account_id:
        account = manager.data.account_dict[account_id]
    else:
        account = manager.new_account(user_id, group_id)
    return f"你还有 {account.bank[prop.id]} 个{prop.name}"


@plugin.handle(["我的信息", "我的资料卡"], ["user_id", "group_id", "nickname"])
async def _(event: Event):
    user, account = manager.account(event)
    info = []
    lines = []
    if DEBUG_MARKING.N(user, account):
        lines.append(debug_marking)
    if CLOVERS_MARKING.N(user, account):
        lines.append(clovers_marking)
    if REVOLUTION_MARKING.N(user, account):
        lines.append(revolution_marking)

    info.append(
        avatar_card(
            await download_url(user.avatar_url),
            account.name or user.name or user.id,
            lines,
        )
    )
    lines = []
    for marking_prop in manager.marking_library.values():
        if count := marking_prop.N(user, account):
            lines.append(f"[color][{marking_prop.color}]Lv.{min(count, 99)}[nowrap][passport]\n[pixel][160]{marking_prop.tip}")
    if lines:
        info.append(text_to_image("\n".join(lines)))
    lines = []
    sum_std_n = user.bank[STD_GOLD.id]
    dist: list[tuple[int, str]] = [(sum_std_n, "个人账户")]
    for group_id, account_id in user.accounts_map.items():
        group = manager.data.group(group_id)
        std_n = manager.data.account_dict[account_id].bank[GOLD.id] * group.level
        if std_n > 0:
            dist.append((std_n, group.nickname))
        sum_std_n += std_n
    lines.append(f"[color][#FFCC33]金币 {format_number(sum_std_n)}")
    lines.append(f"[color][#0066CC]股票 {format_number(manager.stock_value(user.invest))}")
    info.append(text_to_image("\n".join(lines), 40, canvas=dist_card(dist)))
    data = manager.invest_data(user.invest)
    if data:
        info.append(invest_card(data, "股票信息"))
    lines = []
    if account.sign_in is None:
        delta_days = 1
    else:
        delta_days = (datetime.today() - account.sign_in).days
    if delta_days == 0:
        lines.append("[color][green]本群今日已签到")
    else:
        lines.append(f"[color][red]本群连续{delta_days}天 未签到")
    lines += user.message
    info.append(text_to_image("\n".join(lines) + endline("Message"), 30, autowrap=True))
    user.message.clear()
    return manager.info_card(info, event.user_id)


@plugin.handle(["我的道具"], ["user_id", "group_id", "nickname"])
async def _(event: Event):
    user, account = manager.account(event)
    props = Counter()
    props += user.bank
    props += account.bank
    if not props:
        return "您的仓库空空如也。"

    data = manager.props_data(props)
    if len(data) < 10 or event.single_arg() in {"信息", "介绍", "详情"}:
        info = bank_card(data)
    else:
        info = [prop_card(data)]
    return manager.info_card(info, event.user_id)


@plugin.handle(["股票查询", "投资查询"], ["user_id", "group_id", "nickname"])
async def _(event: Event):
    user, account = manager.account(event)
    data = manager.invest_data(user.invest)
    if data:
        return manager.info_card([invest_card(data, f"股票信息:{account.name}")], event.user_id)
    return "您的仓库空空如也。"


@plugin.handle(["群金库"], ["user_id", "group_id", "permission"])
async def _(event: Event):
    if event.is_private() or not (args := event.args_parse()):
        return
    command, n = args[:2]
    if len(command) < 2:
        return
    user_id = event.user_id
    group_id = event.group_id
    group = manager.data.group(group_id)
    if command == "查看":
        data = manager.props_data(group.bank)
        if len(data) < 6:
            info = bank_card(data)
        else:
            info = [prop_card(data, "群金库")]
        data = manager.invest_data(group.invest)
        if data:
            info.append(invest_card(data, "群投资"))
        return manager.info_card(info, user_id) if info else "群金库是空的"
    sign, name = command[0], command[1:]
    match sign:
        case "存":
            sign = 1
        case "取":
            sign = -1
        case _:
            return
    if n < 0:
        n = -n
        sign = -sign
    user, account = manager.locate_account(user_id, group_id)
    if item := manager.props_library.get(name):
        bank_in = group.bank
        bank_out = item.locate_bank(user, account)
    elif (group := manager.group_library.get(name)) and (item := group.stock):
        bank_in = group.invest
        bank_out = user.invest
    else:
        return
    if sign == 1:
        sender = "你"
        receiver = "群金库"
    else:
        if not event.permission:
            return f"你的权限不足。"
        bank_out, bank_in = bank_in, bank_out
        sender = "群金库"
        receiver = "你"
    if bank_n := item.deal(bank_out, -n):
        return f"{command}失败，{sender}还有{bank_n}个{item.name}。"
    item.deal(bank_in, n)
    return f"{sender}向{receiver}转移了{n}个{item.name}"


@plugin.handle(["群资料卡"], ["user_id", "group_id", "nickname"])
async def _(event: Event):
    """
    群资料卡
    """
    group_name = event.single_arg()
    if group_name and not (group := manager.group_library.get(group_name)):
        return f"未找到【{group_name}】"
    else:
        group = manager.data.group(event.group_id)
    info = []
    lines = [
        f"{datetime.fromtimestamp(stock.time).strftime('%Y年%m月%d日')if (stock :=group.stock) else '未发行'}",
        f"等级 {group.level}",
        f"成员 {len(group.accounts_map)}",
    ]
    info.append(avatar_card(await download_url(avatar_url) if (avatar_url := group.avatar_url) else None, group.nickname, lines))
    if ranklist := group.extra.get("revolution_achieve"):
        ranklist = list(ranklist.items())
        ranklist.sort(key=lambda x: x[1], reverse=True)

        def result(user_id, n):
            account_id = group.accounts_map.get(user_id)
            if account_id and (account := manager.data.account_dict.get(account_id)):
                nickname = nickname if len(nickname := account.name) < 7 else nickname[:6] + ".."
            else:
                nickname = "已注销"
            return f"{nickname}[nowrap]\n[right]{n}次"

        info.append(text_to_image("\n".join(result(*seg) for seg in ranklist[:10]) + endline("路灯挂件榜")))
    if record := group.extra.get("stock_record"):
        info.append(candlestick((9.5, 3), 12, record))
    if data := manager.props_data(group.bank):
        info.append(prop_card(data, "群金库"))
    if data := manager.invest_data(group.invest):
        info.append(invest_card(data, "群投资"))
    if group.message:
        info.append(text_to_image("\n".join(group.message) + endline("Message"), 30, autowrap=True))
        group.message.clear()
    return manager.info_card(info, event.user_id)


# 超管指令
@plugin.handle(["获取"], ["user_id", "group_id", "nickname", "permission"], rule=Rule.superuser)
async def _(event: Event):
    if not (args := event.args_parse()):
        return
    name, N = args[:2]
    prop = manager.props_library.get(name)
    if not prop:
        return f"没有【{name}】这种道具。"
    if n := prop.deal(prop.locate_bank(*manager.account(event)), N):
        return f"获取失败，你的【{prop.name}】（{n}））数量不足。"
    return f"你获得了{N}个【{prop.name}】！"


@plugin.handle(["冻结资产"], ["user_id", "group_id", "permission", "at"], rule=[Rule.superuser, Rule.at])
async def _(event: Event):
    user_id = event.user_id
    group_id = event.group_id
    user, account = manager.locate_account(event.at[0], group_id)
    confirm = "".join(str(random.randint(0, 9)) for _ in range(4))

    @plugin.temp_handle(f"{confirm} {user_id} {group_id}", extra_args=["user_id", "group_id", "permission"])
    async def _(temp_event: Event, finish):
        if temp_event.user_id != user_id or temp_event.group_id != group_id:
            return
        finish()
        if temp_event.raw_command != confirm:
            return "【冻结】已取消。"
        bank = Counter()
        bank += user.bank
        user.bank.clear()
        accounts_map = user.accounts_map.copy()
        for _group_id, account_id in accounts_map.items():
            account = manager.data.account_dict[account_id]
            level = manager.data.group(_group_id).level
            bank += {k: v * level for k, v in account.bank.items()}
            manager.data.cancel_account(account_id)
        info = []
        if data := [(prop, v) for k, v in bank.items() if (prop := manager.props_library.get(k))]:
            info.append(prop_card(data, "已删除道具"))
        if data := [(stock, v) for k, v in user.invest.items() if (group := manager.group_library.get(k)) and (stock := group.stock)]:
            info.append(invest_card(data, "已删除股票"))
        user.invest.clear()
        if info:
            return ["冻结完成", manager.info_card(info, user_id)]
        return "冻结完成,目标没有任何资产"

    return f"您即将冻结 {account.name}（{user.id}），请输入{confirm}来确认。"



from io import BytesIO
from PIL import Image, ImageDraw
from PIL.Image import Image as IMG

from clovers_leafgame.output import font_manager, format_number


def draw_rank(data: list[tuple[str, int, bytes]]) -> IMG:
    """
    排名信息
    """
    first = data[0][1]
    canvas = Image.new("RGBA", (880, 80 * len(data) + 20))
    draw = ImageDraw.Draw(canvas)
    y = 20
    i = 1
    font = font_manager.font(40)
    circle_mask = Image.new("RGBA", (60, 60), (255, 255, 255, 0))
    ImageDraw.Draw(circle_mask).ellipse(((0, 0), (60, 60)), fill="black")
    for nickname, v, avatar in data:
        if avatar:
            avatar = Image.open(BytesIO(avatar)).resize((60, 60))
            canvas.paste(avatar, (5, y), circle_mask)
        draw.rectangle(((70, y + 10), (70 + int(v / first * 790), y + 50)), fill="#99CCFFCC")
        draw.text((80, y + 10), f"{i}.{nickname} {format_number(v)}", fill=(0, 0, 0), font=font)
        y += 80
        i += 1
    return canvas



import heapq
import asyncio
from collections import Counter
from collections.abc import Callable
from clovers_leafgame.core.clovers import Event
from clovers_leafgame.main import plugin, manager
from clovers.utils.tools import download_url
from .output import draw_rank


def rank_title(title) -> Callable[[str], int] | None:
    match title:
        case "胜场":
            return lambda locate_id: manager.data.extra.setdefault("win", Counter())[locate_id]
        case "连胜":
            return lambda locate_id: manager.data.extra.setdefault("win_achieve", Counter())[locate_id]
        case "败场":
            return lambda locate_id: manager.data.extra.setdefault("lose", Counter())[locate_id]
        case "连败":
            return lambda locate_id: manager.data.extra.setdefault("lose_achieve", Counter())[locate_id]


def all_ranklist(title: str):
    """总排名"""
    prop = manager.props_library.get(title)
    if not prop:
        return
    ranklist = []
    if prop.domain == 1:
        prop_num = lambda account_id, prop_id: manager.data.account_dict[account_id].bank[prop_id]
        group_level = lambda group_id: manager.data.group(group_id).level
        std_value = lambda prop_id, group_id, account_id: prop_num(account_id, prop_id) * group_level(group_id)
        value = lambda prop_id: sum(std_value(prop_id, *args) for args in user.accounts_map.items())
    else:
        value = lambda prop_id: user.bank[prop_id]
    for user_id in manager.data.user_dict.keys():
        user = manager.data.user(user_id)
        ranklist.append((user.name or user_id, value(prop.id), user.avatar_url))
    return ranklist


def group_ranklist(title: str, group_id: str):
    """群内排名"""
    prop = manager.props_library.get(title)
    if not prop:
        return
    group = manager.data.group_dict[group_id]
    ranklist = []
    for user_id in group.accounts_map.keys():
        user, account = manager.locate_account(user_id, group_id)
        ranklist.append((account.name or user.name or user_id, prop.N(user, account), user.avatar_url))
    return ranklist


@plugin.handle(r"^(.+)排行(.*)", ["user_id", "group_id", "to_me"])
async def _(event: Event):
    title = event.args[0]
    if title.startswith("路灯挂件"):
        counter = Counter()
        for group in manager.data.group_dict.values():
            if not (revolution_achieve := group.extra.get("revolution_achieve")):
                continue
            counter.update(revolution_achieve)
        ranklist = []
        for user_id, value in counter.items():
            user = manager.data.user(user_id)
            ranklist.append((user.name or user_id, value, user.avatar_url))
    elif value := rank_title(title):
        ranklist = []
        for user_id in manager.data.user_dict.keys():
            user = manager.data.user(user_id)
            ranklist.append((user.name or user_id, value(user_id), user.avatar_url))
    elif title.endswith("总"):
        ranklist = all_ranklist(title[:-1])
    else:
        group_name = event.args[1] or event.group_id or manager.data.user(event.user_id).connect
        group = manager.group_library.get(group_name)
        group_id = group.id if group else None
        if not group_id:
            return
        ranklist = group_ranklist(title, group_id)
    if not ranklist:
        return f"无数据，无法进行{title}排行" if event.to_me else None
    ranklist = heapq.nlargest(20, ranklist, key=lambda x: x[1])
    nickname_data = []
    rank_data = []
    task_list = []
    for nickname, v, avatar_url in ranklist[:20]:
        nickname_data.append(nickname)
        rank_data.append(v)
        task_list.append(download_url(avatar_url))
    avatar_data = await asyncio.gather(*task_list)
    return manager.info_card([draw_rank(list(zip(nickname_data, rank_data, avatar_data)))], event.user_id)



from io import BytesIO
from collections.abc import AsyncGenerator
from clovers.core.plugin import Plugin, Result
from clovers.core.config import config as clovers_config
from clovers_leafgame.core.clovers import Event
from .manager import Manager
from .config import Config

config_key = __package__
config_data = Config.model_validate(clovers_config.get(config_key, {}))
"""主配置类"""
clovers_config[config_key] = config_data.model_dump()


def build_result(result):
    if isinstance(result, str):
        return Result("text", result)
    if isinstance(result, BytesIO):
        return Result("image", result)
    if isinstance(result, list):
        return Result("list", [build_result(seg) for seg in result])
    if isinstance(result, AsyncGenerator):

        async def output():
            async for x in result:
                yield build_result(x)

        return Result("segmented", output())
    return result


plugin = Plugin(build_event=lambda event: Event(event), build_result=build_result)
"""小游戏插件实例"""

manager = Manager(config_data.main_path)
"""小游戏管理器实例"""



from typing import Dict
from pydantic import BaseModel
from io import BytesIO
from PIL import Image, ImageDraw, ImageFont
import math
import random

"""
ATK:对士兵攻击力
ATK_T:对建筑攻击力
DEF:对建筑防御值提升
"""

font = ImageFont.truetype(font=fontname, size=30, encoding="utf-8")
font_small = ImageFont.truetype(font=fontname, size=22, encoding="utf-8")


class Castle(BaseModel):
    """
    城池属性
    """

    HP: int = 10
    user_id: str = None
    army: Dict[int, int] = {}
    tower_L: int = 0
    tower_R: int = 0
    line_L: int = 0
    line_R: int = 0
    attack: Dict[int, int] = {}
    soldier = {
        1: {"name": "兵", "ATK": 1, "ATK_T": 1},
        2: {"name": "超", "ATK": 5, "ATK_T": 5},
        3: {"name": "卫", "ATK": 1, "ATK_T": 1},
        4: {"name": "爆", "ATK": 1, "ATK_T": 5},
    }

    cards_table: list = (
        ["超级兵*1"] * 5
        + ["兵*3"] * 10
        + ["兵*2"] * 8
        + ["兵*1"] * 18
        + ["核心血量+5"] * 2
        + ["防线*1（防御力+1）"] * 3
        + ["塔*1（塔血+2）"] * 3
        + ["兵力*2"] * 1
        + ["卫兵*1"] * 2
        + ["爆破兵*1"] * 2
    )

    def turntable(self):
        result = random.choices(self.cards_table, k=4)
        for x in result:
            if x == "超级兵*1":
                self.army[2] = self.army.get(2, 0) + 1
            elif x == "兵*3":
                self.army[1] = self.army.get(1, 0) + 3
            elif x == "兵*2":
                self.army[1] = self.army.get(1, 0) + 2
            elif x == "兵*1":
                self.army[1] = self.army.get(1, 0) + 1
            elif x == "核心血量+5":
                self.HP += 5
            elif x == "防线*1（防御力+1）":
                if self.line_L == 0:
                    self.line_L = 10
                elif self.line_R == 0:
                    self.line_R = 10
                else:
                    if random.randint(0, 1) == 0:
                        self.line_L += 1
                    else:
                        self.line_R += 1
            elif x == "塔*1（塔血+2）":
                if self.tower_L == 0:
                    self.tower_L = 5
                elif self.tower_R == 0:
                    self.tower_R = 5
                else:
                    if random.randint(0, 1) == 0:
                        self.tower_L += 2
                    else:
                        self.tower_R += 2
            elif x == "兵力*2":
                self.army = {k: v * 2 for k, v in self.army.items()}
            elif x == "卫兵*1":
                self.army[3] = self.army.get(3, 0) + 1
            elif x == "爆破兵*1":
                self.army[4] = self.army.get(4, 0) + 1

        return "\n".join(result)


class Player(BaseModel):
    """
    玩家属性
    """

    group_account: GroupAccount = None
    team: str = None
    color: str = None
    castle_ids: list = []


class World(BaseModel):
    """
    世界属性
    """

    start: int = 0
    castles: Dict[str, Castle] = {}
    players: Dict[str, Player] = {}
    ids = []
    teams: list = []
    team_color: list = [
        "#cc0000",
        "#351c75",
        "#e69138",
        "#0b5394",
        "#f1c232",
        "#1155cc",
        "#6aa84f",
        "#134f5c",
        "#45818e",
        "#38761d",
        "#3c78d8",
        "#bf9000",
        "#3d85c6",
        "#b45f06",
    ]
    act = 0
    round = 0
    attack: tuple = ()
    event: dict = None

    def draw(self):
        """
        绘制地图
        """
        # 创建画布
        width, height = 880, 880
        image = Image.new("RGB", (width, height), "#FFFFFF")
        draw = ImageDraw.Draw(image)
        # 圆心坐标和半径
        center_x, center_y = width // 2, height // 2
        radius = 280
        # 协约线
        if self.round < 5:
            inner_outline = "black"
        else:
            inner_outline = "#CCCCCC"

        draw.ellipse((center_x - radius, center_y - radius, center_x + radius, center_y + radius), outline=inner_outline, width=4)

        radius = 380
        # 计算每个等分的角度
        num_slices = 28
        angle_per_slice = 360 / num_slices
        coordinate = {}
        for index, i in enumerate(range(0, num_slices, 2)):
            angle = math.radians((i - 0.7) * angle_per_slice - 90)
            cos = math.cos(angle)
            sin = math.sin(angle)
            x1 = center_x + radius * cos
            y1 = center_y + radius * sin
            xL_T = center_x + (radius - 100) * cos
            yL_T = center_y + (radius - 100) * sin
            angle = math.radians((i + 0.7) * angle_per_slice - 90)
            cos = math.cos(angle)
            sin = math.sin(angle)
            x4 = center_x + radius * cos
            y4 = center_y + radius * sin
            xR_T = center_x + (radius - 100) * cos
            yR_T = center_y + (radius - 100) * sin
            angle = math.radians(i * angle_per_slice - 90)
            cos = math.cos(angle)
            sin = math.sin(angle)
            x2 = x1 + 40 * cos
            y2 = y1 + 40 * sin
            x3 = x4 + 40 * cos
            y3 = y4 + 40 * sin

            index += 1
            castle = self.castles.get(index)
            if castle and (user_id := castle.user_id):
                color = self.players[user_id].color
                color_TL = color if castle.tower_L > 1 else "#CCCCCC"
                color_TR = color if castle.tower_R > 1 else "#CCCCCC"
                color_LL = color if castle.line_L > 1 else "#CCCCCC"
                color_LR = color if castle.line_R > 1 else "#CCCCCC"
            else:
                color = "#CCCCCC"
                color_TL = "#CCCCCC"
                color_TR = "#CCCCCC"
                color_LL = "#CCCCCC"
                color_LR = "#CCCCCC"

            draw.polygon([(x1, y1), (x2, y2), (x3, y3), (x4, y4)], outline="gray", width=1, fill=color)
            draw.line([(x1, y1), (xL_T, yL_T)], width=1, fill=color_LL)
            draw.line([(x4, y4), (xR_T, yR_T)], width=1, fill=color_LR)
            draw.ellipse((xL_T - 10, yL_T - 10, xL_T + 10, yL_T + 10), outline="gray", fill=color_TL)
            draw.ellipse((xR_T - 10, yR_T - 10, xR_T + 10, yR_T + 10), outline="gray", fill=color_TR)

            if castle:
                inner_x = center_x + (radius - 140) * cos
                inner_y = center_x + (radius - 140) * sin
                coordinate[index] = (inner_x, inner_y)
                index = str(index)
                inner_x -= font.getlength(index) // 2
                inner_y -= font.size // 2
                draw.text((inner_x, inner_y), index, fill="black", font=font)
                HP = str(castle.HP + 10 * castle.army.get(3, 0))
                inner_x = center_x + (radius + 15) * cos
                inner_y = center_x + (radius + 15) * sin
                inner_x -= font.getlength(HP) // 2
                inner_y -= font.size // 2
                draw.text((inner_x, inner_y), HP, fill="white", font=font)

                tmp = []
                inner_maxx = 0
                inner_maxy = 0
                for k in range(1, 5):
                    if v := castle.army.get(k):
                        msg = f"{castle.soldier[k]['name']}{v}"
                        inner_maxx = max(font_small.getlength(msg), inner_maxx)
                        inner_maxy += font_small.size + 2
                        tmp.append(msg)

                inner_x = center_x + (radius - 50) * cos - inner_maxx // 2
                inner_y = center_x + (radius - 50) * sin - inner_maxy // 2
                for x in tmp:
                    draw.text((inner_x, inner_y), x, fill="red", font=font_small)
                    inner_y += font_small.size + 2

                if castle.tower_L > 1:
                    tower_L = str(castle.tower_L)
                    inner_x = xL_T - font_small.getlength(tower_L) // 2
                    inner_y = yL_T - 11
                    draw.text((inner_x, inner_y), tower_L, fill="white", font=font_small)
                if castle.tower_R > 1:
                    tower_R = str(castle.tower_R)
                    inner_x = xR_T - font_small.getlength(tower_R) // 2
                    inner_y = yR_T - 11
                    draw.text((inner_x, inner_y), tower_R, fill="white", font=font_small)
                if castle.line_L > 1:
                    line_L = str(castle.line_L)
                    inner_x = ((x1 + xL_T) - font_small.getlength(line_L)) // 2
                    inner_y = ((y1 + yL_T) - font_small.size) // 2
                    draw.text((inner_x, inner_y), line_L, fill=color, font=font_small)
                if castle.line_R > 1:
                    line_R = str(castle.line_R)
                    inner_x = ((x4 + xR_T) - font_small.getlength(line_R)) // 2
                    inner_y = ((y4 + yR_T) - font_small.size) // 2
                    draw.text((inner_x, inner_y), line_R, fill=color, font=font_small)

        if self.attack:
            ATK = self.attack[0]
            color = self.players[self.castles[ATK].user_id].color
            DEF = self.attack[1]
            draw.line((coordinate[ATK], coordinate[DEF]), width=2, fill=color)

        output = BytesIO()
        image.save(output, format="png")
        return output

    def add_player(self, group_account: GroupAccount, index: int, team: str):
        """
        增加玩家
        """
        if team not in self.teams:
            self.teams.append(team)
        user_id = group_account.user_id
        self.players[user_id] = Player(
            group_account=group_account, team=team, color=self.team_color[self.teams.index(team)], castle_ids=[index]
        )
        self.ids.append(user_id)
        self.castles[index] = Castle(user_id=user_id)



from datetime import datetime
from pathlib import Path
from pydantic import BaseModel
from collections import Counter

KeyMap = dict[str, str]


class Item(BaseModel):
    id: str
    name: str

    def deal(self, bank: Counter[str], unsettled: int):
        prop_id = self.id
        n = bank[prop_id]
        if unsettled < 0 and n < (-unsettled):
            return n or -1
        bank[prop_id] += unsettled


class User(BaseModel):
    id: str
    name: str = ""
    avatar_url: str = ""
    connect: str = ""
    bank: Counter[str] = Counter()
    invest: Counter[str] = Counter()
    extra: dict = {}
    accounts_map: KeyMap = {}
    """Find account ID from group_id"""
    message: list[str] = []

    def add_message(self, message: str):
        self.message.append(message)
        self.message = self.message[-30:]


class Account(BaseModel):
    user_id: str
    group_id: str
    name: str = ""
    sign_in: datetime | None = None
    bank: Counter[str] = Counter()
    extra: dict = {}

    @property
    def id(self):
        return f"{self.user_id}-{self.group_id}"


class Prop(Item):
    rare: int
    """稀有度"""
    domain: int
    """
    作用域   
        0:无(空气)
        1:群内
        2:全局
    """
    flow: int
    """
    道具时效
        0:时效道具
        1:永久道具
    """
    number: int
    """道具编号"""
    color: str = "black"
    intro: str = ""
    tip: str = ""

    def __init__(self, id: str, **data) -> None:
        data.update({"id": id, "rare": int(id[0]), "domain": int(id[1]), "flow": int(id[2]), "number": int(id[3:])})
        super().__init__(**data)

    def locate_bank(self, user: User, account: Account):
        match self.domain:
            case 1:
                return account.bank
            case _:
                return user.bank

    def N(self, user: User, account: Account):
        return self.locate_bank(user, account)[self.id]


class Stock(Item):
    value: int = 0
    """全群资产"""
    floating: float = 0
    """浮动资产"""
    issuance: int = 0
    """股票发行量"""
    time: float
    """注册时间"""
    exchange: dict[str, tuple[int, float]] = {}
    """交易信息"""


class Group(BaseModel):
    id: str
    name: str | None = None
    avatar_url: str | None = None
    level: int = 1
    stock: Stock | None = None
    bank: Counter[str] = Counter()
    invest: Counter[str] = Counter()
    extra: dict = {}
    accounts_map: KeyMap = {}
    """Find account ID from user_id"""
    message: list[str] = []

    @property
    def nickname(self):
        return self.stock.name if self.stock else self.name or self.id


class DataBase(BaseModel):
    user_dict: dict[str, User] = {}
    group_dict: dict[str, Group] = {}
    account_dict: dict[str, Account] = {}
    extra: dict = {}

    @classmethod
    def load(cls, file: Path):
        if file.exists():
            with open(file, "r", encoding="utf8") as f:
                data = cls.model_validate_json(f.read())
        else:
            data = cls()
        return data

    def register(self, account: Account):
        """注册个人账户"""
        user_id = account.user_id
        group_id = account.group_id
        account_id = account.id
        self.user(user_id).accounts_map[group_id] = account_id
        self.group(group_id).accounts_map[user_id] = account_id
        self.account_dict[account_id] = account

    def user(self, user_id: str):
        if user_id not in self.user_dict:
            self.user_dict[user_id] = User(id=user_id)

        return self.user_dict[user_id]

    def group(self, group_id: str):
        if group_id not in self.group_dict:
            self.group_dict[group_id] = Group(id=group_id)

        return self.group_dict[group_id]

    def cancel_account(self, account_id: str):
        """注销 account"""
        account = self.account_dict.get(account_id)
        if not account:
            return
        user_id = account.user_id
        group_id = account.group_id
        del self.account_dict[account_id]
        try:
            del self.user_dict[user_id].accounts_map[group_id]
        except Exception as e:
            print(e)
        try:
            del self.group_dict[group_id].accounts_map[user_id]
        except Exception as e:
            print(e)

    def cancel_user(self, user_id: str):
        """注销 user"""
        user = self.user_dict.get(user_id)
        if not user:
            return
        del self.user_dict[user_id]
        for group_id, account_id in user.accounts_map.items():
            try:
                del self.account_dict[account_id]
            except Exception as e:
                print(e)
            try:
                del self.group_dict[group_id].accounts_map[user_id]
            except Exception as e:
                print(e)

    def cancel_group(self, group_id: str):
        """注销 group"""
        group = self.group_dict.get(group_id)
        del self.group_dict[group_id]
        if not group:
            return
        for user_id, account_id in group.accounts_map.items():
            try:
                del self.account_dict[account_id]
            except Exception as e:
                print(e)
            try:
                del self.user_dict[user_id].accounts_map[group_id]
            except Exception as e:
                print(e)



from datetime import datetime
from io import BytesIO
from PIL import Image, ImageDraw
import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
from matplotlib.font_manager import FontProperties
import mplfinance as mpf


from clovers.utils.linecard import FontManager, linecard
from clovers.utils.tools import format_number
from .core.data import Prop, Stock
from .main import config_data

fontname = config_data.fontname
fallback = config_data.fallback_fonts

font_manager = FontManager(fontname, fallback, (30, 40, 60))

plt.rcParams["font.family"] = FontProperties(fname=font_manager.path).get_name()
plt.rcParams["font.sans-serif"] = [FontProperties(fname=path).get_name() for path in font_manager.fallback_paths]


def text_to_image(text: str, font_size=40, width=880, **kwargs):
    return linecard(text, font_manager, font_size, width, **kwargs)


def endline(tip: str) -> str:
    return f"\n----\n[right][color][grey][font][][30]{tip}"


def bank_card(data: list[tuple[Prop, int]]):
    data.sort(key=lambda x: x[0].rare)

    def result(prop: Prop, n: int):
        quant = {0: "天", 1: "个"}[prop.flow]
        return linecard(
            (
                f"[font][][60][color][{prop.color}]【{prop.name}】[nowrap][passport]\n"
                f"[right]{format_number(n)}{quant}\n"
                f"----\n{prop.intro.replace('\n','[passport]\n')}"
                f"\n[right]{prop.tip.replace('\n','[passport]\n')}"
            ),
            font_manager,
            40,
            width=880,
            autowrap=True,
        )

    return [result(*args) for args in data]


def prop_card(data: list[tuple[Prop, int]], tip: str | None = None):
    data.sort(key=lambda x: x[0].rare)

    def result(prop: Prop, n: int):
        quant = {0: "天", 1: "个"}.get(prop.flow)
        return (
            f"[color][{prop.color}]{prop.name}[nowrap][passport]\n"
            f"[pixel][350]{prop.rare*'☆'}[nowrap][passport]\n"
            f"[right]{format_number(n)}{quant}"
        )

    info = "\n".join(result(*args) for args in data)
    if tip:
        info += endline(tip)
    return linecard(info, font_manager, 40, spacing=1.5, width=880)


def invest_card(data: list[tuple[Stock, int]], tip: str | None = None):
    def result(stock: Stock, n: int):
        issuance = stock.issuance
        buy = format_number(max(stock.floating, stock.value) / issuance) if issuance else "未发行"
        sell = format_number(stock.floating / issuance) if issuance else "未发行"
        return (
            f"[pixel][20]{stock.name}\n"
            f"[pixel][20]数量 [nowrap]\n[color][{'green' if n else 'red'}]{n}[nowrap]\n"
            f"[pixel][280]购买 [nowrap]\n[color][{'red' if buy > sell else 'green'}]{buy}[nowrap]\n"
            f"[pixel][580]结算 [nowrap]\n[color][green]{sell}[nowrap]\n"
        )

    info = "\n".join(result(*args) for args in data)
    if tip:
        info += endline(tip)
    return linecard(info, font_manager, 40, width=880)


AVATAR_MASK = Image.new("RGBA", (260, 260), (255, 255, 255, 0))
ImageDraw.Draw(AVATAR_MASK).ellipse(((0, 0), (260, 260)), fill="black")


def avatar_card(avatar: bytes | None, nickname: str, lines: list[str]):
    font = font_manager.font(40)
    canvas = Image.new("RGBA", (880, 300))
    if avatar:
        canvas.paste(Image.open(BytesIO(avatar)).resize((260, 260)), (20, 20), AVATAR_MASK)
    draw = ImageDraw.Draw(canvas)
    draw.text((300, 40), f"{nickname}", fill=(0, 0, 0), font=font)
    draw.line(((300, 120), (860, 120)), fill="gray", width=4)
    for n, line in enumerate(lines):
        draw.text((300, 140 + n * 50), "•", fill="gray", font=font)
        draw.text((840, 140 + n * 50), "•", fill="gray", font=font)
        x = 340
        for char in line:
            draw.text((x, 140 + n * 50), char, fill="gray", font=font)
            x += 40

    return canvas


def candlestick(figsize: tuple[float, float], length: int, history: list[tuple[float, float]]):
    """
    生成股价K线图
        figsize:图片尺寸
        length:OHLC采样长度
        history:历史数据
    """
    t, price = zip(*history)
    l = len(t)
    t = [t[i : i + length] for i in range(0, l, length)]
    price = [price[i : i + length] for i in range(0, l, length)]
    D, O, H, L, C = [], [], [], [], []
    for i in range(len(price)):
        D.append(datetime.fromtimestamp(t[i][0]))
        O.append(price[i][0])
        H.append(max(price[i]))
        L.append(min(price[i]))
        C.append(price[i][-1])
    data = pd.DataFrame({"date": D, "open": O, "high": H, "low": L, "close": C})
    data = data.set_index("date")
    style = mpf.make_mpf_style(
        base_mpf_style="charles",
        marketcolors=mpf.make_marketcolors(up="#006340", down="#a02128", edge="none"),
        y_on_right=False,
        facecolor="#FFFFFF99",
        figcolor="none",
    )
    output = BytesIO()
    mpf.plot(
        data,
        type="candlestick",
        xlabel="",
        ylabel="",
        datetime_format="%H:%M",
        tight_layout=True,
        style=style,
        figsize=figsize,
        savefig=output,
    )
    return Image.open(output)


def dist_card(
    dist: list[tuple[int, str]],
    colors=[
        "#351c75",
        "#0b5394",
        "#1155cc",
        "#134f5c",
        "#38761d",
        "#bf9000",
        "#b45f06",
        "#990000",
        "#741b47",
    ],
):
    dist.sort(key=lambda x: x[0], reverse=True)
    labels = []
    x = []
    sum_value = sum(d[0] for d in dist)
    limit = 0.01 * sum_value
    for n, (value, name) in enumerate(dist):
        if n < 8 and value > limit:
            x.append(value)
            labels.append(name)
        else:
            labels.append("其他")
            x.append(sum(seg[0] for seg in dist[n:]))
            break
    n += 1
    output = BytesIO()

    plt.figure(figsize=(6.6, 3.4))
    plt.pie(
        np.array(x),
        labels=[""] * n,
        autopct=lambda pct: "" if pct < 1 else f"{pct:.1f}%",
        colors=colors[0:n],
        wedgeprops={"edgecolor": "none"},
        textprops={"fontsize": 15},
        pctdistance=1.2,
        explode=[0, 0.1, 0.19, 0.27, 0.34, 0.40, 0.45, 0.49, 0.52][0:n],
    )
    plt.legend(labels, loc=(-0.6, 0), frameon=False)
    plt.axis("equal")
    plt.subplots_adjust(top=0.95, bottom=0.05, left=0.4, hspace=0, wspace=0)
    plt.savefig(output, format="png", dpi=100, transparent=True)
    plt.close()
    canvas = Image.new("RGBA", (880, 340))
    canvas.paste(Image.open(output), (220, 0))
    return canvas



import random


def random_poker(n: int = 1, range_point: tuple[int, int] = (1, 14)):
    """
    生成随机牌库
    """
    poker_deck = [(suit, point) for suit in range(1, 5) for point in range(*range_point)]
    poker_deck = poker_deck * n
    random.shuffle(poker_deck)
    return poker_deck


poker_suit = {4: "♠", 3: "♥", 2: "♣", 1: "♦"}
poker_point = {
    1: " A",
    2: " 2",
    3: " 3",
    4: " 4",
    5: " 5",
    6: " 6",
    7: " 7",
    8: " 8",
    9: " 9",
    10: "10",
    11: " J",
    12: " Q",
    13: " K",
    14: " A",
}


def poker_show(hand: list[tuple[int, int]], split: str = ""):
    return split.join(f"【{poker_suit[suit]}{poker_point[point]}】" for suit, point in hand)



from pydantic import BaseModel


class Config(BaseModel):
    # 重置冷却时间，设置为0禁用发起重置
    revolt_cd: int = 28800
    # 重置的基尼系数
    revolt_gini: float = 0.68
    # 统计基尼系数只计算金币数大于下面参数的用户
    gini_filter_gold: int = 1000
    # 重置签到的范围
    revolt_gold: tuple[int, int] = (1000, 2000)
    # 注册公司金币数
    company_public_gold: int = 20000



from pathlib import Path
import json
from .horse import Event
from clovers.core.logger import logger


def load_dlcs(resource_path: Path = Path(__file__).parent / "event_library") -> list[Event]:
    events_list = []
    files = resource_path.iterdir()
    for x in files:
        log = f"加载事件文件：{x.name}"
        try:
            with open(x, "r", encoding="utf-8") as f:
                events_list += [deal(event) for event in json.load(f) if event]
            logger.info(f"{log} 成功！")
        except:
            logger.warn(f"{log} 失败...")

    return events_list


def deal(raw_event: dict):
    raw_event["only_key"] = raw_event.get("race_only")
    buffs = set()
    buff_tags = ["locate_lock", "vertigo", "hiding"]
    for buff_tag in buff_tags:
        if raw_event.get(buff_tag) == 1:
            buffs.add(buff_tag)
    for buff_tag in raw_event.get("other_buff", []):
        buffs.add(buff_tag)
    raw_event["buffs"] = buffs
    return Event.model_validate(raw_event)
